

ProfessionsCustomerListingsElementMixin = CreateFromMixins(ScrollListLineMixin, TableBuilderRowMixin);

function ProfessionsCustomerListingsElementMixin:OnLineEnter()
	self.HighlightTexture:Show();
end

function ProfessionsCustomerListingsElementMixin:OnLineLeave()
	self.HighlightTexture:Hide();
end

function ProfessionsCustomerListingsElementMixin:Init(elementData)
	self.order = elementData.option;
end

local function AnyRecraftablePredicate(itemGUID)
	local craftRecipeID, craftSkillLineAbility = C_TradeSkillUI.GetOriginalCraftRecipeID(itemGUID);
	return craftSkillLineAbility ~= nil and C_CraftingOrders.CanOrderSkillAbility(craftSkillLineAbility);
end

local ProfessionsCustomerOrderFormEvents =
{
    "UNIT_INVENTORY_CHANGED",
	"BAG_UPDATE",
	"CRAFTINGORDERS_ORDER_PLACEMENT_RESPONSE",
	"CRAFTINGORDERS_ORDER_CANCEL_RESPONSE",
};

ProfessionsCustomerOrderFormMixin = {};

function ProfessionsCustomerOrderFormMixin:InitPaymentContainer()
	self.PaymentContainer.TipMoneyInputFrame:SetOnValueChangedCallback(GenerateClosure(self.UpdateTotalPrice, self));
	self.PaymentContainer.TipMoneyInputFrame.CopperBox:Hide();
	
	self.PaymentContainer.ListOrderButton:SetScript("OnClick", function()
		self:ListOrder();
	end);

	self.PaymentContainer.CancelOrderButton:SetScript("OnClick", function()
		C_CraftingOrders.CancelOrder(self.order.orderID);
	end);
end

function ProfessionsCustomerOrderFormMixin:SetRecraftItemGUID(itemGUID)
	local craftRecipeID, skillLineAbilityID = C_TradeSkillUI.GetOriginalCraftRecipeID(itemGUID);
	self.order.spellID = craftRecipeID;
	self.order.skillLineAbilityID = skillLineAbilityID;
	self.recraftGUID = itemGUID;
	self:InitSchematic();
	self:SetupQualityDropDown();
	self:UpdateMinimumQuality();
end

function ProfessionsCustomerOrderFormMixin:InitButtons()
	self.RecraftSlot.InputSlot:SetScript("OnMouseDown", function(button, buttonName, down)
		if buttonName == "LeftButton" and not self.committed then
			local flyout = ToggleProfessionsItemFlyout(self.RecraftSlot.InputSlot, ProfessionsCustomerOrdersFrame);
			if flyout then
				local function OnFlyoutItemSelected(o, flyout, elementData)
					local itemLocation = C_Item.GetItemLocation(elementData.itemGUID);
					C_Sound.PlayItemSound(Enum.ItemSoundType.Drop, itemLocation);
					self:SetRecraftItemGUID(elementData.itemGUID);
				end
	
				flyout.GetElementsImplementation = function(self)
					local isIndexTable = true;
					local itemGUIDs = tFilter(C_TradeSkillUI.GetRecraftItems(), AnyRecraftablePredicate, isIndexTable);
					local items = ItemUtil.TransformItemGUIDsToItems(itemGUIDs);
					local elementData = {items = items, itemGUIDs = itemGUIDs};
					return elementData;
				end

				flyout.OnElementEnterImplementation = function(elementData, tooltip)
					tooltip:SetItemByGUID(elementData.itemGUID);
				end
				
				flyout.OnElementEnabledImplementation = function(button, elementData)
					return true;
				end

				local cannotFilter = true;
				flyout:Init(self.RecraftSlot.InputSlot, self.transaction, cannotFilter);
				flyout:RegisterCallback(ProfessionsItemFlyoutMixin.Event.ItemSelected, OnFlyoutItemSelected, self);
			end
		end
	end);

	self.RecraftSlot.InputSlot:SetScript("OnEnter", function()
		GameTooltip:SetOwner(self.RecraftSlot.InputSlot, "ANCHOR_RIGHT");

		local itemGUID = self.transaction and self.transaction:GetRecraftAllocation();
		if itemGUID then
			GameTooltip:SetItemByGUID(itemGUID);
			if not self.committed then
				GameTooltip_AddBlankLineToTooltip(GameTooltip);
				GameTooltip_AddInstructionLine(GameTooltip, RECRAFT_REAGENT_TOOLTIP_CLICK_TO_REPLACE);
			end
		elseif self.order.recraftItemHyperlink then
			GameTooltip:SetHyperlink(self.order.recraftItemHyperlink);
		else
			GameTooltip_AddInstructionLine(GameTooltip, RECRAFT_REAGENT_TOOLTIP_CLICK_TO_ADD);
		end
		GameTooltip:Show();
	end);

	self.RecraftSlot.OutputSlot:SetScript("OnEnter", function()
		local itemGUID = self.transaction and self.transaction:GetRecraftAllocation();
		if itemGUID or self.committed then
			GameTooltip:SetOwner(self.RecraftSlot.OutputSlot, "ANCHOR_RIGHT");

			local optionalReagents = self.transaction:CreateOptionalCraftingReagentInfoTbl();
			local minimumQuality = self.minQualityIDs and self.minQualityIDs[self.order.minQuality];
			if itemGUID then
				GameTooltip:SetRecipeResultItem(self.order.spellID, optionalReagents, itemGUID, nilRecipeLevel, minimumQuality);
			else
				GameTooltip:SetRecipeResultItemForOrder(self.order.spellID, optionalReagents, self.order.orderID, nilRecipeLevel, minimumQuality);
			end
		end
	end);

	self.PaymentContainer.ViewListingsButton:SetScript("OnClick", function() self:ShowCurrentListings(); end);
end

function ProfessionsCustomerOrderFormMixin:InitCurrentListings()
	self.CurrentListings:SetTitle(PROFESSIONS_CURRENT_LISTINGS);
	self.CurrentListings.CloseButton:SetScript("OnClick", function() self:HideCurrentListings(); end);

	self.CurrentListings.SetSortOrder = function(currentListings, sortOrder)
		if currentListings.primarySort.order == sortOrder then
			currentListings.primarySort.ascending = not currentListings.primarySort.ascending;
		else
			currentListings.secondarySort = CopyTable(currentListings.primarySort);
			currentListings.primarySort =
			{
				order = sortOrder;
				ascending = true;
			};
		end
	
		if currentListings.tableBuilder then
			for frame in currentListings.tableBuilder:EnumerateHeaders() do
				frame:UpdateArrow();
			end
		end
	
		self:RequestCurrentListings();
	end

	self.CurrentListings.primarySort =
	{
		order = ProfessionsSortOrder.Tip;
		ascending = false;
	};

	self.CurrentListings.secondarySort =
	{
		order = ProfessionsSortOrder.Reagents;
		ascending = true;
	};

	self.CurrentListings.GetSortOrder = function(currentListings)
		return currentListings.primarySort.order, currentListings.primarySort.ascending;
	end

	local pad = 5;
	local spacing = 1;
	local view = CreateScrollBoxListLinearView(pad, pad, pad, pad, spacing);
	view:SetElementInitializer("ProfessionsCustomerListingsElementTemplate", function(button, elementData)
		button:Init(elementData);
	end);
	ScrollUtil.InitScrollBoxListWithScrollBar(self.CurrentListings.OrderList.ScrollBox, self.CurrentListings.OrderList.ScrollBar, view);

	self.CurrentListings.tableBuilder = CreateTableBuilder(nil, ProfessionsTableBuilderMixin);
	local function ElementDataTranslator(elementData)
		return elementData;
	end;
	ScrollUtil.RegisterTableBuilder(self.CurrentListings.OrderList.ScrollBox, self.CurrentListings.tableBuilder, ElementDataTranslator);

	local function ElementDataProvider(elementData)
		return elementData;
	end;
	self.CurrentListings.tableBuilder:SetDataProvider(ElementDataProvider);

	local PTC = ProfessionsTableConstants;
	self.CurrentListings.tableBuilder:Reset();
	self.CurrentListings.tableBuilder:SetColumnHeaderOverlap(2);
	self.CurrentListings.tableBuilder:SetHeaderContainer(self.CurrentListings.OrderList.HeaderContainer);
	self.CurrentListings.tableBuilder:SetTableMargins(5, 5);
	self.CurrentListings.tableBuilder:SetTableWidth(PTC.Tip.Width + PTC.Reagents.Width);

	self.CurrentListings.tableBuilder:AddFixedWidthColumn(self.CurrentListings, PTC.NoPadding, PTC.Tip.Width, PTC.Tip.LeftCellPadding,
										  	PTC.Tip.RightCellPadding, ProfessionsSortOrder.Tip, "ProfessionsCrafterTableCellActualCommissionTemplate");
	self.CurrentListings.tableBuilder:AddFixedWidthColumn(self.CurrentListings, PTC.NoPadding, PTC.Reagents.Width, PTC.Reagents.LeftCellPadding,
										  	PTC.Reagents.RightCellPadding, ProfessionsSortOrder.Reagents, "ProfessionsCrafterTableCellReagentsTemplate");
	self.CurrentListings.tableBuilder:Arrange();

	local function OnDataRangeChanged(sortPending, indexBegin, indexEnd)
		if (not self.expectMoreRows) or (self.requestCallback ~= nil) or (not self.numOrders) then
			return;
		end

		local ordersFromBottom = self.numOrders - indexEnd;
		local requestMoreOrdersThreshold = 30;
		if ordersFromBottom < requestMoreOrdersThreshold then
			self:RequestMoreOrders();
		end
	end
	self.CurrentListings.OrderList.ScrollBox:RegisterCallback(ScrollBoxListMixin.Event.OnDataRangeChanged, OnDataRangeChanged, self);
end

function ProfessionsCustomerOrderFormMixin:OnLoad()
	local function PoolReset(pool, slot)
		slot:Reset();
		slot.Button:SetScript("OnEnter", nil);
		slot.Button:SetScript("OnClick", nil);
		slot.Button:SetScript("OnMouseDown", nil);
		FramePool_HideAndClearAnchors(pool, slot);
	end
	self.reagentSlotPool = CreateFramePool("FRAME", self, "ProfessionsReagentSlotTemplate", PoolReset);

	self:InitPaymentContainer();
	self:InitButtons();
	self:InitCurrentListings();
end

function ProfessionsCustomerOrderFormMixin:OnShow()
    FrameUtil.RegisterFrameForEvents(self, ProfessionsCustomerOrderFormEvents);
end

function ProfessionsCustomerOrderFormMixin:OnHide()
    FrameUtil.UnregisterFrameForEvents(self, ProfessionsCustomerOrderFormEvents);
	if self.requestCallback then
		self.requestCallback:Cancel();
		self.requestCallback = nil;
	end

	self:HideCurrentListings();
end

function ProfessionsCustomerOrderFormMixin:OnEvent(event, ...)
	if event == "UNIT_INVENTORY_CHANGED" or event == "BAG_UPDATE" then
		self:UpdateReagentSlots();
	elseif event == "CRAFTINGORDERS_ORDER_PLACEMENT_RESPONSE" or event == "CRAFTINGORDERS_ORDER_CANCEL_RESPONSE" then
		local result = ...;
		local success = result == Enum.CraftingOrderResult.Ok;
		if success then
			self:Hide();
			CallMethodOnNearestAncestor(self, "SelectMode", ProfessionsCustomerOrdersMode.Orders);
		else
			local errorText;
			if event == "CRAFTINGORDERS_ORDER_PLACEMENT_RESPONSE" then
				errorText = PROFESSIONS_ORDER_PLACEMENT_FAILED;
			elseif event == "CRAFTINGORDERS_ORDER_CANCEL_RESPONSE" then
				errorText = (result == Enum.CraftingOrderResult.AlreadyClaimed) and PROFESSIONS_ORDER_CANCEL_FAILED_CLAIMED or PROFESSIONS_ORDER_CANCEL_FAILED;
			end
			UIErrorsFrame:AddExternalErrorMessage(errorText);
		end
	end
end

function ProfessionsCustomerOrderFormMixin:SetupQualityDropDown()
	if not self.minQualityIDs then
		return;
	end

	local function Initializer(dropDown, level)
		local function DropDownButtonClick(button)
			self:SetMinimumQualityIndex(button.value);
		end
		
		for index = 1, #(self.minQualityIDs) do
			local info = UIDropDownMenu_CreateInfo();
			info.fontObject = Number12Font;
			info.text = index == 1 and NONE or CreateAtlasMarkup(string.format("Professions-ChatIcon-Quality-Tier%d", index), 15, 15, 0, -1);
			info.value = index;
			info.checked = nil;
			info.func = DropDownButtonClick;
			UIDropDownMenu_AddButton(info);
		end
	end

	UIDropDownMenu_SetWidth(self.MinimumQualityDropDown, 70);
	self.MinimumQualityDropDown.Text:SetJustifyH("CENTER");
	UIDropDownMenu_Initialize(self.MinimumQualityDropDown, Initializer);
	UIDropDownMenu_SetSelectedValue(self.MinimumQualityDropDown, self.order.minQuality);
end

function ProfessionsCustomerOrderFormMixin:SetupOrderRecipientDropDown()
	local function Initializer(dropDown, level)
		local function DropDownButtonClick(button)
			self:SetOrderRecipient(button.value);
		end
	
		local recipientTypes = {};
		table.insert(recipientTypes, { text = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_PUBLIC, value = Enum.CraftingOrderType.Public, });
		if IsInGuild() then
			table.insert(recipientTypes, { text = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_GUILD, value = Enum.CraftingOrderType.Guild, });
		end
		table.insert(recipientTypes, { text = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_PRIVATE, value = Enum.CraftingOrderType.Personal, });

		for _, type in ipairs(recipientTypes) do
			local info = UIDropDownMenu_CreateInfo();
			info.fontObject = GameFontHighlightSmall;
			info.text = type.text;
			info.minWidth = 108;
			info.value = type.value;
			info.checked = nil;
			info.func = DropDownButtonClick;
			UIDropDownMenu_AddButton(info);
		end
	end

	UIDropDownMenu_SetWidth(self.OrderRecipientDropDown, 136);
	UIDropDownMenu_JustifyText(self.OrderRecipientDropDown, "LEFT");
	UIDropDownMenu_Initialize(self.OrderRecipientDropDown, Initializer);
	UIDropDownMenu_SetSelectedValue(self.OrderRecipientDropDown, self.order.orderType);

	self:UpdateMinimumQuality();
end

function ProfessionsCustomerOrderFormMixin:SetupDurationDropDown()
	self.PaymentContainer.Duration:SetText(PROFESSIONS_CRAFTING_FORM_CUSTOMER_DURATION);

	local function Initializer(dropDown, level)
		local function DropDownButtonClick(button)
			self:SetDuration(button.value);
		end

		local durationTypes = 
		{
			{ text = PROFESSIONS_LISTING_DURATION_ONE, value = Enum.CraftingOrderDuration.Short, },
			{ text = PROFESSIONS_LISTING_DURATION_TWO, value = Enum.CraftingOrderDuration.Medium, },
			{ text = PROFESSIONS_LISTING_DURATION_THREE, value = Enum.CraftingOrderDuration.Long, },
		};
		
		for _, duration in ipairs(durationTypes) do
			local info = UIDropDownMenu_CreateInfo();
			info.fontObject = GameFontHighlightSmall;
			info.text = duration.text;
			info.minWidth = 108;
			info.value = duration.value;
			info.checked = nil;
			info.func = DropDownButtonClick;
			UIDropDownMenu_AddButton(info);
		end
	end

	UIDropDownMenu_Initialize(self.PaymentContainer.DurationDropDown, Initializer);
	UIDropDownMenu_SetSelectedValue(self.PaymentContainer.DurationDropDown, self.duration);
end

function ProfessionsCustomerOrderFormMixin:UpdateMinimumQuality()
	if self.committed then
		return;
	end

	if not self.minQualityIDs then
		self.MinimumQuality:Hide();
		self.MinimumQualityDropDown:Hide();
		return;
	end

	self.MinimumQuality:Show();
	self.MinimumQuality:SetWidth(250);

	if self.order.orderType == Enum.CraftingOrderType.Public then
		self.MinimumQuality:SetText(PROFESSIONS_CRAFTING_FORM_MIN_QUALITY_ANY);
		self.MinimumQualityDropDown:Hide();

		self.order.minQuality = 1;
	else
		self.MinimumQuality:Show();
		self.MinimumQuality:SetText(PROFESSIONS_CRAFTING_FORM_MIN_QUALITY);
		self.MinimumQualityDropDown:Show();
	end

	self.MinimumQuality:SetWidth(self.MinimumQuality:GetStringWidth());
end

function ProfessionsCustomerOrderFormMixin:UpdateDepositCost()
	if self.committed or not self.order.skillLineAbilityID then
		return;
	end

	self.PaymentContainer.PostingFee:Show();
	self.PaymentContainer.PostingFeeMoneyDisplayFrame:Show();

	self.depositCost = C_CraftingOrders.CalculateCraftingOrderPostingFee(self.order.skillLineAbilityID, self.order.orderType, self.duration);
	self.PaymentContainer.PostingFeeMoneyDisplayFrame:SetAmount(self.depositCost);
	self:UpdateTotalPrice();
end

function ProfessionsCustomerOrderFormMixin:SetDuration(index)
	self.duration = index;

	Professions.SetDefaultOrderDuration(index);

	UIDropDownMenu_SetSelectedValue(self.PaymentContainer.DurationDropDown, index);
	
	self:UpdateDepositCost();
end

function ProfessionsCustomerOrderFormMixin:SetOrderRecipient(index)
	self.order.orderType = index;

	Professions.SetDefaultOrderRecipient(index);

	UIDropDownMenu_SetSelectedValue(self.OrderRecipientDropDown, index);

	self:UpdateMinimumQuality();
	self:UpdateDepositCost();

	self.OrderRecipientTarget:SetShown(index == Enum.CraftingOrderType.Personal);

	self:UpdateListOrderButton();
end

function ProfessionsCustomerOrderFormMixin:SetMinimumQualityIndex(index)
	self.order.minQuality = index;

	UIDropDownMenu_SetSelectedValue(self.MinimumQualityDropDown, index);
end

function ProfessionsCustomerOrderFormMixin:UpdateReagentSlots()
	if not self.transaction then
		return;
	end

	local transaction = self.transaction;
	local recipeID = transaction:GetRecipeID();
	local recipeSchematic = transaction:GetRecipeSchematic();
	local committed = self.order.orderID ~= nil;

	self.hasAllRequiredReagents = true;

	self.reagentSlotPool:ReleaseAll();
	local reagentTypes = {};

	local slotParents =
	{
		[Enum.CraftingReagentType.Basic] = self.ReagentContainer.Reagents,
		[Enum.CraftingReagentType.Optional] = self.ReagentContainer.OptionalReagents,
	};

	for slotIndex, reagentSlotSchematic in ipairs(recipeSchematic.reagentSlotSchematics) do
		local reagentType = reagentSlotSchematic.reagentType;
		if reagentType ~= Enum.CraftingReagentType.Finishing then
			local slots = reagentTypes[reagentType];
			if not slots then
				slots = {};
				reagentTypes[reagentType] = slots;
			end

			local hasAnyAllocation = transaction:HasAllocations(slotIndex);

			local slot = self.reagentSlotPool:Acquire();
			table.insert(slots, slot);

			slot:SetParent(slotParents[reagentType]);

			slot:Init(transaction, reagentSlotSchematic);
			slot:Show();

			if committed then
				if not hasAnyAllocation then
					slot:SetOverrideNameColor(DISABLED_REAGENT_COLOR);
				end
			end

			if reagentType == Enum.CraftingReagentType.Basic then
				local canAllocate = reagentSlotSchematic.orderSource ~= Enum.CraftingOrderReagentSource.Crafter and not committed and Professions.CanAllocateReagents(transaction, slotIndex);
				slot:SetHighlightShown(canAllocate or (reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Customer and not committed));

				if canAllocate then
					local useBestQuality = true;
					Professions.AllocateBasicReagents(transaction, slotIndex, useBestQuality);
					slot:Update();
				end

				local canToggle = reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Any and not committed;
				slot:SetCheckboxShown(canToggle);
				if canToggle then
					slot:SetCheckboxChecked(canAllocate);
					slot:SetCheckboxEnabled(canAllocate);
					slot:SetUnallocatable(not canAllocate);
					slot:SetCheckboxCallback(function(provided)
						slot:SetHighlightShown(provided);
						if not provided then
							transaction:ClearAllocations(slotIndex);

							slot.Button:UpdateCursor();
							slot:Update();
						else
							local useBestQuality = true;
							Professions.AllocateBasicReagents(transaction, slotIndex, useBestQuality);
							slot:Update();
						end

						slot:SetUnallocatable(not provided);
					end);
					if not canAllocate then
						slot:SetCheckboxTooltipText(ERROR_COLOR:WrapTextInColorCode(PROFESSIONS_ORDERS_NOT_ENOUGH_REAGENTS));
					end
				else
					if reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Crafter then
						slot:SetOverrideNameColor(DISABLED_REAGENT_COLOR);
						slot:SetShowOnlyRequired(true);
					elseif reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Customer then
						if not canAllocate and not committed then
							slot:SetOverrideNameColor(ERROR_COLOR);
							self.hasAllRequiredReagents = false;
						end
					end
				end

				if Professions.GetReagentInputMode(reagentSlotSchematic) == Professions.ReagentInputMode.Quality then
					slot.Button:SetScript("OnClick", function(button, buttonName, down)
						if IsShiftKeyDown() then
							local qualityIndex = Professions.FindFirstQualityAllocated(transaction, reagentSlotSchematic) or 1;
							local handled, link = Professions.HandleQualityReagentItemLink(recipeID, reagentSlotSchematic, qualityIndex);
							if not handled then
								Professions.TriggerReagentClickedEvent(link);
							end
							return;
						end

						if not slot:IsUnallocatable() then
							if buttonName == "LeftButton" then
								local function OnAllocationsAccepted(dialog, allocations, reagentSlotSchematic)
									transaction:OverwriteAllocations(reagentSlotSchematic.slotIndex, allocations);

									slot:Update();
								end

								self.QualityDialog:RegisterCallback(ProfessionsQualityDialogMixin.Event.Accepted, OnAllocationsAccepted, slot);
								
								local allocationsCopy = transaction:GetAllocationsCopy(slotIndex);
								self.QualityDialog:Open(recipeID, reagentSlotSchematic, allocationsCopy, slotIndex);
							end
						end
					end);

					slot.Button:SetScript("OnEnter", function()
						GameTooltip:SetOwner(slot.Button, "ANCHOR_RIGHT");
						Professions.SetupQualityReagentTooltip(slot, transaction);
						if reagentSlotSchematic.orderSource ~= Enum.CraftingOrderReagentSource.Any then
							GameTooltip_AddBlankLineToTooltip(GameTooltip);
							local sourceMsg = (reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Crafter) and PROFESSIONS_ORDER_CRAFTER_REQUIRED_REAGENT or PROFESSIONS_ORDER_CUSTOMER_REQUIRED_REAGENT;
							GameTooltip_AddHighlightLine(GameTooltip, sourceMsg);
						end
						GameTooltip:Show();
					end);
				else
					slot.Button:SetScript("OnClick", function(button, buttonName, down)
						if IsShiftKeyDown() then
							local handled, link = Professions.HandleFixedReagentItemLink(recipeID, reagentSlotSchematic);
							if not handled then
								Professions.TriggerReagentClickedEvent(link);
							end
							return;
						end
					end);

					slot.Button:SetScript("OnEnter", function()
						GameTooltip:SetOwner(slot.Button, "ANCHOR_RIGHT");
						GameTooltip:SetRecipeReagentItem(recipeID, reagentSlotSchematic.dataSlotIndex);
						if reagentSlotSchematic.orderSource ~= Enum.CraftingOrderReagentSource.Any then
							GameTooltip_AddBlankLineToTooltip(GameTooltip);
							local sourceMsg = (reagentSlotSchematic.orderSource == Enum.CraftingOrderReagentSource.Crafter) and PROFESSIONS_ORDER_CRAFTER_REQUIRED_REAGENT or PROFESSIONS_ORDER_CUSTOMER_REQUIRED_REAGENT;
							GameTooltip_AddHighlightLine(GameTooltip, sourceMsg);
						end
						GameTooltip:Show();
					end);
				end

				if committed then
					slot:SetCheckmarkShown(hasAnyAllocation);
					slot:SetCheckmarkTooltipText(hasAnyAllocation and PROFESSIONS_CUSTOMER_ORDER_REAGENT_PROVIDED);
					slot:SetOverrideQuantity(hasAnyAllocation and reagentSlotSchematic.quantityRequired or 0);
				end
			else
				slot.Button:SetLocked(false);

				slot.Button:SetScript("OnEnter", function()
					GameTooltip:SetOwner(slot.Button, "ANCHOR_RIGHT");
					local suppressInstruction = committed;
					Professions.SetupOptionalReagentTooltip(slot, recipeID, reagentType, reagentSlotSchematic.slotInfo.slotText, nil, nil, suppressInstruction);
					GameTooltip:Show();
				end);
				
				slot.Button:SetScript("OnMouseDown", function(button, buttonName, down)
					if not slot:IsUnallocatable() then
						if buttonName == "LeftButton" then
							local flyout = ToggleProfessionsItemFlyout(slot.Button, ProfessionsCustomerOrdersFrame);
							if flyout then
								local function OnFlyoutItemSelected(o, flyout, elementData)
									local item = elementData.item;
									local reagent = Professions.CreateCraftingReagentByItemID(item:GetItemID());
									transaction:OverwriteAllocation(slotIndex, reagent, reagentSlotSchematic.quantityRequired);

									slot:SetItem(item);
									slot:SetHighlightShown(true);
								end

								flyout.GetElementsImplementation = function(self, filterOwned)
									local itemIDs = Professions.ExtractItemIDsFromCraftingReagents(reagentSlotSchematic.reagents);
									if filterOwned then
										itemIDs = ItemUtil.FilterOwnedItems(itemIDs);
									end
									local items = ItemUtil.TransformItemIDsToItems(itemIDs);
									local elementData = {items = items};
									return elementData;
								end
								
								flyout.OnElementEnterImplementation = function(elementData, tooltip)
									Professions.FlyoutOnElementEnterImplementation(elementData, tooltip, recipeID);
								end
	
								local cannotFilter = false;
								flyout:Init(slot.Button, self.transaction, cannotFilter);
								flyout:RegisterCallback(ProfessionsItemFlyoutMixin.Event.ItemSelected, OnFlyoutItemSelected, slot);
							end
						elseif buttonName == "RightButton" then
							transaction:ClearAllocations(slotIndex);

							slot:ClearItem();
							slot.Button.InputOverlay.AddIcon:Show();
							slot:SetHighlightShown(false);
						end
					end
				end);
			end
		end
	end
	
	if committed then
		for slot in self.reagentSlotPool:EnumerateActive() do
			slot:SetUnallocatable(true);
			slot.Button.InputOverlay.AddIcon:Hide();
		end
	end

	Professions.LayoutReagentSlots(
		reagentTypes[Enum.CraftingReagentType.Basic], self.ReagentContainer.Reagents,
		reagentTypes[Enum.CraftingReagentType.Optional], self.ReagentContainer.OptionalReagents);

	self:UpdateListOrderButton();
end

function ProfessionsCustomerOrderFormMixin:InitSchematic()
	self.ReagentContainer.Reagents:Show();
	self.ReagentContainer.OptionalReagents:Show();
	self.PaymentContainer.ViewListingsButton:Enable();

	local recipeID = self.order.spellID;
	local recipeSchematic = self.order.spellID and C_TradeSkillUI.GetRecipeSchematic(self.order.spellID, self.order.isRecraft);
	self.transaction = recipeSchematic and CreateProfessionsRecipeTransaction(recipeSchematic);

	if self.order.isRecraft then
		if self.recraftGUID then
		self.transaction:SetRecraftAllocation(self.recraftGUID);
		else
			self.transaction:SetRecraftAllocationOrderID(self.order.orderID);
		end
	end
	self.RecraftSlot:Init(self.transaction, AnyRecraftablePredicate, function(itemGUID) self:SetRecraftItemGUID(itemGUID); end, self.order.recraftItemHyperlink);

	self.minQualityIDs = recipeID and C_TradeSkillUI.GetQualitiesForRecipe(recipeID);

	if not self.committed then
		Professions.AllocateAllBasicReagents(self.transaction, true);
	else
		for _, reagentInfo in ipairs(self.order.reagents) do
			local allocations = self.transaction:GetAllocations(reagentInfo.reagentSlot);
			allocations:Allocate(reagentInfo.reagent, reagentInfo.reagent.quantity);
		end
	end

	if not self.order.isRecraft then
		self.loader = CreateProfessionsRecipeLoader(recipeSchematic, function()
			local reagents = nil;
			local outputItemInfo = C_TradeSkillUI.GetRecipeOutputItemData(recipeID, reagents);
			if outputItemInfo.hyperlink then
				local item = Item:CreateFromItemLink(outputItemInfo.hyperlink);
				self.RecipeName:SetText(item:GetItemName());
				self.RecipeName:SetTextColor(item:GetItemQualityColorRGB());
			else
				self.RecipeName:SetText(self.recipeSchematic.name);
				self.RecipeName:SetTextColor(NORMAL_FONT_COLOR:GetRGB());
			end

			self.RecipeName:SetHeight(self.RecipeName:GetStringHeight());

			Professions.SetupOutputIcon(self.OutputIcon, self.transaction, outputItemInfo);
		end);

		self.OutputIcon:SetScript("OnEnter", function()
			GameTooltip:SetOwner(self.OutputIcon, "ANCHOR_RIGHT");
			local optionalReagents = self.transaction:CreateOptionalCraftingReagentInfoTbl();
			local minimumQuality = self.minQualityIDs and self.minQualityIDs[self.order.minQuality];
			self.OutputIcon:SetScript("OnUpdate", function()
				GameTooltip:SetRecipeResultItem(recipeSchematic.recipeID, optionalReagents, nilRecraftAllocation, nilRecipeLevel, minimumQuality);
			end);
		end);
		self.OutputIcon:SetScript("OnLeave", function()
			GameTooltip_Hide();
			self.OutputIcon:SetScript("OnUpdate", nil);
		end);
	end

	self:UpdateReagentSlots();
	self:UpdateMinimumQuality();
	self:UpdateDepositCost();
end

function ProfessionsCustomerOrderFormMixin:Init(order)
	self.order = order;
	if order.minQuality < 1 then
		order.minQuality = 1;
	end

	self.duration = Professions.GetDefaultOrderDuration();
	self.depositCost = 0;
	self.committed = order.orderID ~= nil;
	self.transaction = nil;
	self.RecraftSlot:Init(self.transaction, AnyRecraftablePredicate, function(itemGUID) self:SetRecraftItemGUID(itemGUID); end, self.order.recraftItemHyperlink);
	self.minQualityIDs = nil;
	self.recraftGUID = nil;
	if self.loader then
		self.loader:Cancel();
		self.loader = nil;
	end
	self:HideCurrentListings();

	self.OutputIcon:SetShown(not order.isRecraft);
	self.RecipeName:SetShown(not order.isRecraft);
	self.RecraftSlot:SetShown(order.isRecraft);
	local minQualityXOfs = order.isRecraft and 120 or 0;
	local minQualityYOfs = order.isRecraft and 15 or -5;
	self.MinimumQuality:ClearAllPoints();
	self.MinimumQuality:SetPoint("TOPLEFT", self.RecipeName, "BOTTOMLEFT", minQualityXOfs, minQualityYOfs);

	local editBox = self.PaymentContainer.NoteEditBox.ScrollingEditBox;
	editBox:SetDefaultTextEnabled(not self.committed);
	editBox:SetText(order.customerNotes or "");
	editBox:SetEnabled(not self.committed);

	for _, region in ipairs(self.uncommittedRegions) do
		region:SetShown(not self.committed);
	end
	for _, region in ipairs(self.PaymentContainer.uncommittedRegions) do
		region:SetShown(not self.committed);
	end

	for _, region in ipairs(self.committedRegions) do
		region:SetShown(self.committed);
	end
	for _, region in ipairs(self.PaymentContainer.committedRegions) do
		region:SetShown(self.committed);
	end

	if self.order.spellID then
		self:InitSchematic();
	else
		self.ReagentContainer.Reagents:Hide();
		self.ReagentContainer.OptionalReagents:Hide();
		self.MinimumQuality:Hide();
		self.MinimumQualityDropDown:Hide();
		self.PaymentContainer.PostingFee:Hide();
		self.PaymentContainer.PostingFeeMoneyDisplayFrame:Hide();
		self.PaymentContainer.ViewListingsButton:Disable();
	end

	if not self.committed then
		self:SetupQualityDropDown();
		self:SetupOrderRecipientDropDown();
		self:SetupDurationDropDown();
		self:UpdateTotalPrice();

		self.OrderRecipientTarget:SetShown(self.order.orderType == Enum.CraftingOrderType.Personal);
		self.PaymentContainer.TipMoneyInputFrame:SetAmount(order.tipAmount);

		self.ReagentContainer.Reagents.Label:SetText(PROFESSIONS_CUSTOMER_REAGENT_CONTAINER_LABEL);
		self.ReagentContainer.OptionalReagents.Label:SetText(PROFESSIONS_CUSTOMER_OPTIONAL_REAGENT_CONTAINER_LABEL);
	else
		self.PaymentContainer.TipMoneyDisplayFrame:SetAmount(order.tipAmount);

		local remainingTime = Professions.GetCraftingOrderRemainingTime(order.expirationTime);
		local noSeconds = true;
		self.PaymentContainer.TimeRemainingDisplay.Text:SetText(SecondsToTime(remainingTime, noSeconds));

		self.OrderRecipientDisplay.Crafter:SetShown(order.crafterName);
		self.OrderRecipientDisplay.CrafterValue:SetText(order.crafterName or "");
		local orderTypeText;
		if self.order.orderType == Enum.CraftingOrderType.Public then
			orderTypeText = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_PUBLIC;
		elseif self.order.orderType == Enum.CraftingOrderType.Guild then
			orderTypeText = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_GUILD;
		elseif self.order.orderType == Enum.CraftingOrderType.Personal then
			orderTypeText = PROFESSIONS_CRAFTING_FORM_ORDER_RECIPIENT_PRIVATE;
		end
		self.OrderRecipientDisplay.PostedToValue:SetText(orderTypeText);
		local rightRecipientTextWidth = math.max(self.OrderRecipientDisplay.PostedToValue:GetStringWidth(), self.OrderRecipientDisplay.CrafterValue:GetStringWidth());
		self.OrderRecipientDisplay.PostedToValue:SetWidth(rightRecipientTextWidth);
		self.OrderRecipientDisplay.CrafterValue:SetWidth(rightRecipientTextWidth);

		self:UpdateCancelOrderButton();

		self.ReagentContainer.Reagents.Label:SetText(PROFESSIONS_PROVIDED_REAGENT_CONTAINER_LABEL);
		self.ReagentContainer.OptionalReagents.Label:SetText(PROFESSIONS_PROVIDED_OPTIONAL_REAGENT_CONTAINER_LABEL);

		local hasQuality = self.minQualityIDs ~= nil;
		self.MinimumQuality:SetShown(hasQuality);
		if hasQuality then
			self.MinimumQuality:SetWidth(250);
			self.MinimumQuality:SetText(PROFESSIONS_CRAFTING_FORM_MIN_REQUIRED_QUALITY:format(order.minQuality <= 1 and NONE or CreateAtlasMarkup(string.format("Professions-ChatIcon-Quality-Tier%d", order.minQuality), 15, 15, 0, -1)));
			self.MinimumQuality:SetWidth(self.MinimumQuality:GetStringWidth());
		end
	end
end

function ProfessionsCustomerOrderFormMixin:UpdateTotalPrice()
	if self.committed then
		return;
	end

	local tip = self.PaymentContainer.TipMoneyInputFrame:GetAmount();
	local total = self.depositCost + tip;
	self.PaymentContainer.TotalPriceMoneyDisplayFrame:SetAmount(total);
	self:UpdateListOrderButton();
end

function ProfessionsCustomerOrderFormMixin:UpdateListOrderButton()
	if self.committed then
		return;
	end

	local listOrderButton = self.PaymentContainer.ListOrderButton;

	local enabled = true;
	local errorText;

	if self.order.isRecraft and not self.order.skillLineAbilityID then
		enabled = false;
		errorText = PROFESSIONS_MUST_SELECT_RECRAFT_TARGET;
	elseif not self.hasAllRequiredReagents then
		enabled = false;
		errorText = PROFESSIONS_INSUFFICIENT_REAGENTS_CUSTOMER;
	elseif self.order.orderType == Enum.CraftingOrderType.Personal and self.OrderRecipientTarget:GetText() == "" then
		enabled = false;
		errorText = PRFOESSIONS_MISSING_ORDER_TARGET;
	elseif self.PaymentContainer.TipMoneyInputFrame:GetAmount() <= 0 then
		enabled = false;
		errorText = PROFESSIONS_ORDER_MUST_TIP;
	elseif self.PaymentContainer.TotalPriceMoneyDisplayFrame:GetAmount() > GetMoney() then
		enabled = false;
		self.errorText = NOT_ENOUGH_GOLD;
	end

	listOrderButton:SetEnabled(enabled);

	if errorText then
		listOrderButton:SetScript("OnEnter", function()
			GameTooltip:SetOwner(listOrderButton, "ANCHOR_RIGHT");
			GameTooltip_AddErrorLine(GameTooltip, errorText);
			GameTooltip:Show();
		end);
	else
		listOrderButton:SetScript("OnEnter", nil);
	end
end

function ProfessionsCustomerOrderFormMixin:UpdateCancelOrderButton()
	local enabled = true;
	local errorText;

	if self.order.orderState ~= Enum.CraftingOrderState.Created then
		enabled = false;
		errorText = PROFESSIONS_ORDER_CANT_CANCEL_CLAIMED;
	end

	self.PaymentContainer.CancelOrderButton:SetEnabled(enabled);

	if errorText then
		self.PaymentContainer.CancelOrderButton:SetScript("OnEnter", function()
			GameTooltip:SetOwner(self.PaymentContainer.CancelOrderButton, "ANCHOR_RIGHT");
			GameTooltip_AddErrorLine(GameTooltip, errorText);
			GameTooltip:Show();
		end);
	else
		self.PaymentContainer.CancelOrderButton:SetScript("OnEnter", nil);
	end
end

function ProfessionsCustomerOrderFormMixin:ListOrder()
	local newOrderInfo =
	{
		skillLineAbilityID = self.order.skillLineAbilityID,
		orderType = self.order.orderType,
		orderDuration = self.duration,
		tipAmount = self.PaymentContainer.TipMoneyInputFrame:GetAmount(),
		customerNotes = self.PaymentContainer.NoteEditBox.ScrollingEditBox:GetInputText(),
		reagentItems = self.transaction:CreateRegularReagentInfoTbl(),
		craftingReagentItems = self.transaction:CreateCraftingReagentInfoTbl(),
		recraftItem = self.recraftGUID,
	};

	if self.order.orderType ~= Enum.CraftingOrderType.Public and self.minQualityIDs then
		newOrderInfo.minCraftingQualityID = self.order.minQuality > 1 and self.minQualityIDs[self.order.minQuality] or 0;
	end

	if self.order.orderType == Enum.CraftingOrderType.Personal then
		newOrderInfo.orderTarget = self.OrderRecipientTarget:GetText();
	end

	C_CraftingOrders.PlaceNewOrder(newOrderInfo);
end

local baseFrameWidth = 800;
local currentListingsPopoutFrameWidth = 330;

function ProfessionsCustomerOrderFormMixin:HideCurrentListings()
	self.CurrentListings:Hide();

	local ordersPanel = self:GetParent();
	SetUIPanelAttribute(ordersPanel, "width", baseFrameWidth);
	UpdateUIPanelPositions(ordersPanel);
end

function ProfessionsCustomerOrderFormMixin:ShowCurrentListings()
	self.CurrentListings:Show();

	local ordersPanel = self:GetParent();
	SetUIPanelAttribute(ordersPanel, "width", baseFrameWidth + currentListingsPopoutFrameWidth);
	UpdateUIPanelPositions(ordersPanel);

	self:RequestCurrentListings();
end

function ProfessionsCustomerOrderFormMixin:SendOrderRequest(request)
	self.lastRequest = request;

	if self.requestCallback then
		self.requestCallback:Cancel();
	end
	self.requestCallback = C_FunctionContainers.CreateCallback(function(...) self:OrderRequestCallback(...); end);
	request.callback = self.requestCallback;
	C_CraftingOrders.RequestCustomerOrders(request);
end

function ProfessionsCustomerOrderFormMixin:RequestCurrentListings()
	self.CurrentListings.OrderList.ResultsText:Hide();
	self.CurrentListings.OrderList.LoadingSpinner:Show();
	self.CurrentListings.OrderList.SpinnerAnim:Restart();
	self.CurrentListings.OrderList.ScrollBox:Hide();

	local selectedSkillLineAbility = self.order.skillLineAbilityID;
	local request =
	{
		orderType = Enum.CraftingOrderType.Public,
		selectedSkillLineAbility = selectedSkillLineAbility,
		searchFavorites = false,
		initialNonPublicSearch = false,
		offset = 0,
		forCrafter = false,
		primarySort = Professions.TranslateSearchSort(self.CurrentListings.primarySort),
		secondarySort = Professions.TranslateSearchSort(self.CurrentListings.secondarySort),
	};
	self:SendOrderRequest(request);
end

function ProfessionsCustomerOrderFormMixin:DisplayCurrentListings(offset, isSorted)
	self.CurrentListings.OrderList.LoadingSpinner:Hide();
	self.CurrentListings.OrderList.SpinnerAnim:Stop();
	self.CurrentListings.OrderList.ScrollBox:Show();

	local orders = C_CraftingOrders.GetCustomerOrders();

	if #orders == 0 then
		self.CurrentListings.OrderList.ResultsText:SetText(PROFESSIONS_CUSTOMER_NO_ORDERS);
		self.CurrentListings.OrderList.ResultsText:Show();
	else
		self.CurrentListings.OrderList.ResultsText:Hide();
	end
	if not isSorted then
		table.sort(orders, function(lhs, rhs)
			local res, equal = Professions.ApplySortOrder(self.primarySort.order);
			if not equal then
				if self.primarySort.ascending then
					return res;
				else
					return not res;
				end
			end

			if self.secondarySort then
				res, equal = Professions.ApplySortOrder(self.secondarySort.order);
				if self.secondarySort.ascending then
					return res;
				else
					return equal or (not res);
				end
			end

			return true;
		end);
	end

	local dataProvider;
	if offset == 0 then
		dataProvider = CreateDataProvider();
	for _, order in ipairs(orders) do
		dataProvider:Insert({option = order});
	end
	self.CurrentListings.OrderList.ScrollBox:SetDataProvider(dataProvider);
	else
		dataProvider = self.CurrentListings.OrderList.ScrollBox:GetDataProvider();
		for idx = offset + 1, #orders do
			local order = orders[idx];
			dataProvider:Insert({option = order});
		end
	end
end

function ProfessionsCustomerOrderFormMixin:OrderRequestCallback(result, orderType, displayBuckets, expectMoreRows, offset, isSorted)
	self.expectMoreRows = expectMoreRows;

	self:DisplayCurrentListings(offset, isSorted);

	self.requestCallback = nil;
end

function ProfessionsCustomerOrderFormMixin:RequestMoreOrders()
	if (not self.expectMoreRows) or (not self.lastRequest) or (self.requestCallback ~= nil) then
		return;
	end

	local request = self.lastRequest;
	request.offset = self.numOrders;
	self:SendOrderRequest(request);
end