local isGMClient = IsGMClient()
local PTR_IssueReporter = CreateFrame("Frame", nil, UIParent)

function PTR_IssueReporter.Init()
    Blizzard_PTRIssueReporter_Saved = Blizzard_PTRIssueReporter_Saved or {}
    --------------------------------------------------ALWAYS DISPLAYED BUTTONS--------------------------------------------------
    local IgnoreLocations = {1813, 1814, 1879, 1882, 1883, 1892,  1893, 1897, 1898, 1907}
    PTR_IssueReporter.Body = CreateFrame("Frame", nil, PTR_IssueReporter)
    PTR_IssueReporter.Data = {
        targetQuests = {
            --if this list remains empty it will ask on every quest, otherwise only those specified by ID
        },
        bossKillProbabilityPopup = 100, --from 0 to 100 percent
        throttleSubmitTime = 5, --seconds since last sending a feedback update
        AlertQueueSize = 3,
        QuestHistorySize = 5,
        npcAlertQuestion = "Did you experience any bugs or problems\nwith this enemy?",
        watermark = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Textures\\watermark",
        textureFile = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Textures\\UI-Background-Marble",
        textureFile2 = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Textures\\UI-Background-Marble",
        pushedTexture = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Buttons\\UI-Quickslot-Depress",
        fontString = "GameFontNormal",
        confusedIcon = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Icons\\TutorialFrame-QuestionMark",
        bugreport = "Interface\\Addons\\Blizzard_PTRFeedback\\Assets\\Icons\\HelpIcon-Bug",
        height = 50,
        lastSubmitTime,
        unitToken = "player",
        targetToken = "target",
        alertFrameText = "Did you have any bugs or other problems\nwith this creature?",
        SELF_REPORTED_CONFUSED = 1,
        SELF_REPORTED_BUG = 2,
        BOSS_KILL = 3,
        QUEST_TURNED_IN = 4,
        ISLANDS = 5,
        WARFRONTS = 6,
        MESSAGE_KEY = "[&#@^$M*]",
        Thanks = "Your bug has been received.",
        SubmitText = "Submit Bug",
    }
    PTR_IssueReporter.TriggerEvents = {
        [5] = {enabled = false, label = "Island Expedition", question = "Did you experience any bugs or problems with this Island Expedition?", checkboxes = {"Yes", "No"}},
        [6] = {enabled = false, label = "Warfront", question = "Did you experience any bugs or problems with this Warfront?", checkboxes = {"Yes", "No"}},
    }

    function PTR_IssueReporter.Reminder(enable, ...)
        for k,v in pairs({...}) do
            if (enable) then
                ActionButton_ShowOverlayGlow(v)
            else
                ActionButton_HideOverlayGlow(v)
            end
        end
    end

    function PTR_IssueReporter.CreateIssueButton(name, icon, tooltip, func)
        local scalar = PTR_IssueReporter.Body:GetHeight()
        local newbutton = CreateFrame("Button", name, PTR_IssueReporter)
        newbutton:SetSize(scalar, scalar)
        newbutton:SetHighlightTexture(icon, "ADD")
        newbutton:SetNormalTexture(icon)
        newbutton:SetPushedTexture(PTR_IssueReporter.Data.pushedTexture)
        FramePainter.AddBorder(newbutton)
        newbutton:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
            GameTooltip:SetText(name, 1, 1, 1, true);
            GameTooltip:AddLine(tooltip, nil, nil, nil, true);
            GameTooltip:SetMinimumWidth(100);
            GameTooltip:Show()
        end)
        newbutton:SetScript("OnLeave", function(self)
            self:SetButtonState("NORMAL")
            GameTooltip:Hide()
        end)
        if (func) then
            newbutton:SetScript("OnClick", function(self, button, down)
                func()
            end)
        end
        newbutton:SetScript("OnHide", function(self)
            self:Show()
        end)
        return newbutton
    end

    function PTR_IssueReporter.ExclusiveCheckButton(buttons, index)
        for i=1,#buttons do
            if (i == index) then
                buttons[i]:SetChecked(true)
            else
                buttons[i]:SetChecked(false)
            end
        end
        return index
    end

    function PTR_IssueReporter.SendResults(reportType, ...)
        --reset idle time
        local packageString = PTR_FeedbackDiagnostic:Get()
        local finalMessage = string.format('%s,%s,%s', PTR_IssueReporter.Data.MESSAGE_KEY, reportType, packageString)
        if (reportType == PTR_IssueReporter.Data.BOSS_KILL) then
            local choice, bossName, bossID, bossDifficulty, comments = ...
            bossName = bossName:gsub(",", " ")
            bossDifficulty = bossDifficulty:gsub(",", " ")
            comments = comments:gsub(","," ")
            finalMessage = string.format('%s,%s,%s,%s,%s,%s', finalMessage, choice, bossName, bossID, bossDifficulty, comments)
        elseif (reportType == PTR_IssueReporter.Data.QUEST_TURNED_IN) then
            local choice, questID, comments, history = ...
            comments = comments:gsub(","," ")
            finalMessage = string.format('%s,%s,%s,%s,%s', finalMessage, choice, questID, comments, history)
        elseif (PTR_IssueReporter.TriggerEvents[reportType]) then
            local choice, instanceID, comments = ...
            comments = comments:gsub(","," ")
            finalMessage = string.format('%s,%s,%s,%s', finalMessage, choice, instanceID, comments)
        else
            local comments = ...
            comments = comments:gsub(","," ")
            finalMessage = string.format('%s,%s', finalMessage, comments)
        end

        if (GMSubmitBug) then
            GMSubmitBug(finalMessage)
            UIErrorsFrame:Clear()
        end
        return finalMessage
    end

    --cosmetics
    PTR_IssueReporter:Hide()
    PTR_IssueReporter:SetScript("OnShow", function(self)
        PTR_IssueReporter:ClearAllPoints()
        if (Blizzard_PTRIssueReporter_Saved.x and Blizzard_PTRIssueReporter_Saved.y) then
            PTR_IssueReporter:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", Blizzard_PTRIssueReporter_Saved.x, Blizzard_PTRIssueReporter_Saved.y)
        else
            PTR_IssueReporter:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, UIParent:GetHeight()*0.25)
        end
        PTR_IssueReporter:SetClampRectInsets(0, 0, 0, -PTR_IssueReporter.Body.Texture:GetHeight())
    end)
    PTR_IssueReporter:SetFrameStrata("HIGH")
    PTR_IssueReporter.text = PTR_IssueReporter:CreateFontString("CheckListText", "OVERLAY", PTR_IssueReporter.Data.fontString)
    PTR_IssueReporter.text:SetWidth(PTR_IssueReporter:GetWidth())
    PTR_IssueReporter.text:SetHeight(PTR_IssueReporter:GetHeight())
    PTR_IssueReporter.text:SetPoint("CENTER", PTR_IssueReporter, "CENTER", 0, 0)
    PTR_IssueReporter.text:SetText("Bug\nReporter")
    PTR_IssueReporter:SetSize(PTR_IssueReporter.text:GetStringWidth()*1.5,32)
    FramePainter.AddBorder(PTR_IssueReporter)
    PTR_IssueReporter.Body:SetSize(PTR_IssueReporter:GetWidth(), PTR_IssueReporter.Data.height)
    PTR_IssueReporter.Body:SetPoint("TOP", PTR_IssueReporter, "BOTTOM", 0, PTR_IssueReporter:GetHeight()*0.05)
    FramePainter.AddBackground(PTR_IssueReporter, PTR_IssueReporter.Data.textureFile2)

    PTR_IssueReporter:SetScript("OnEnter", function(self)
        SetCursor("Interface\\CURSOR\\UI-Cursor-Move.blp")
    end)
    PTR_IssueReporter:SetScript("OnLeave", function(self)
        ResetCursor()
    end)

    PTR_IssueReporter:SetScript("OnUpdate", function(self, elapsed)
        PTR_IssueReporter.Data.lastSubmitTime = PTR_IssueReporter.Data.lastSubmitTime or 0
        if (PTR_IssueReporter.Data.lastSubmitTime > 0) then
            PTR_IssueReporter.Data.lastSubmitTime = PTR_IssueReporter.Data.lastSubmitTime - elapsed
        end
    end)

    --information button
    FramePainter.AddInfoButton(PTR_IssueReporter)
    FramePainter.AddTooltip(PTR_IssueReporter.InfoButton,
        "Bug examples",
        "This enemy could not be hit.\nI fell through the ground.\nMy spell isn't doing what it says it should do.\nThere are missing graphics here.")

    --create buttons
    local Confused = PTR_IssueReporter.CreateIssueButton("Confused",
        PTR_IssueReporter.Data.confusedIcon,
        "I'm not sure what I should do\nand/or where I should go.")
    local ReportBug = PTR_IssueReporter.CreateIssueButton("Bug",
        PTR_IssueReporter.Data.bugreport,
        "I have encountered a bug or other technical problem.")
    ReportBug.InputBox = CreateFrame("Frame", "ReportBugInputBox", ReportBug)
    ReportBug.InputBox:SetPoint("CENTER", UIParent, "CENTER")
    ReportBug.InputBox:SetSize(200, 200)
    FramePainter.AddBackground(ReportBug.InputBox, PTR_IssueReporter.Data.watermark);
    FramePainter.AddBorder(ReportBug.InputBox)
    ReportBug.InputBox.InputField = FramePainter.NewEditBox("ReportBugExtraInfo", "BOTTOM", ReportBug.InputBox, "BOTTOM", "", ReportBug.InputBox:GetWidth(), 40)
    ReportBug.InputBox.text = ReportBug.InputBox:CreateFontString("CheckListText", "OVERLAY", PTR_IssueReporter.Data.fontString)
    ReportBug.InputBox.text:SetPoint("TOPLEFT", ReportBug.InputBox, "TOPLEFT", 8, -8)
    ReportBug.InputBox.text:SetPoint("BOTTOMRIGHT", ReportBug.InputBox.InputField, "TOPRIGHT", -8, 8)
    ReportBug.InputBox.text:SetJustifyH("LEFT")
    ReportBug.InputBox.text:SetJustifyV("TOP")
    ReportBug.InputBox.text:SetWordWrap(true)
    ReportBug.InputBox.text:SetNonSpaceWrap(true)
    FramePainter.AddBackground(ReportBug.InputBox.InputField, PTR_IssueReporter.Data.textureFile)
    ReportBug.InputBox.InputField:SetScript("OnTextChanged", function(self, userInput)
        local body = self:GetText()
        ReportBug.InputBox.text:SetText(body)
    end)
    ReportBug.InputBox.SubmitButton = CreateFrame("Button", nil, ReportBug.InputBox, "UIPanelButtonTemplate")
    ReportBug.InputBox.SubmitButton:SetPoint("TOP", ReportBug.InputBox, "BOTTOM", 0, 0)
    ReportBug.InputBox.SubmitButton:SetText(PTR_IssueReporter.Data.SubmitText)
    ReportBug.InputBox.SubmitButton:SetSize(ReportBug.InputBox.SubmitButton:GetTextWidth()*1.5, 32)
    ReportBug.InputBox.SubmitButton:SetScript("OnClick", function(self, button, down)
        local comments = ReportBug.InputBox.text:GetText() or ""
        if (ReportBug.InputBox.Reason == PTR_IssueReporter.Data.SELF_REPORTED_BUG) then
            PTR_IssueReporter.SendResults(PTR_IssueReporter.Data.SELF_REPORTED_BUG, comments)
        else
            PTR_IssueReporter.SendResults(PTR_IssueReporter.Data.SELF_REPORTED_CONFUSED, comments)
        end
        UIErrorsFrame:AddMessage(PTR_IssueReporter.Data.Thanks)
        ReportBug.InputBox:Hide()
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
    end)
    ReportBug.InputBox.Reason = SELF_REPORTED_BUG
    ReportBug.InputBox.Title = CreateFrame("Button", nil, ReportBug.InputBox, "UIPanelButtonTemplate")
    ReportBug.InputBox.Title:SetButtonState("NORMAL", true)
    ReportBug.InputBox.Title:SetNormalTexture(PTR_IssueReporter.Data.textureFile2)
    ReportBug.InputBox.Title:EnableMouse(false)
    ReportBug.InputBox.Title:SetPoint("BOTTOM", ReportBug.InputBox, "TOP", 0, 2)
    ReportBug.InputBox.Title:SetSize(ReportBug.InputBox:GetWidth(), 32)
    FramePainter.AddBorder(ReportBug.InputBox.Title)
    table.insert(UISpecialFrames, ReportBug.InputBox:GetName())
    ReportBug.InputBox:Hide()
    ReportBug.InputBox:SetScript("OnShow", function(self)
        ReportBug.InputBox.InputField:SetText("")
    end)
    ReportBug:SetScript("OnClick", function(self, button, down)
        if (self.InputBox:IsShown()) then

            if (self.InputBox.Reason == PTR_IssueReporter.Data.SELF_REPORTED_CONFUSED) then
                ReportBug.InputBox.Reason = PTR_IssueReporter.Data.SELF_REPORTED_BUG
                self.InputBox.Title:SetText("Bug Report")
            else
                self.InputBox:Hide()
            end
        else
            self.InputBox.Reason = PTR_IssueReporter.Data.SELF_REPORTED_BUG
            self.InputBox.Title:SetText("Bug Report")
            self.InputBox:Show()
            self.InputBox.InputField:SetFocus()
        end
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
    end)

    Confused:SetScript("OnClick", function(self, button, down)
        if (ReportBug.InputBox:IsShown()) then
            if (ReportBug.InputBox.Reason == PTR_IssueReporter.Data.SELF_REPORTED_BUG) then

                ReportBug.InputBox.Reason = PTR_IssueReporter.Data.SELF_REPORTED_CONFUSED
                ReportBug.InputBox.Title:SetText("Confused Report")
            else
                ReportBug.InputBox:Hide()
            end
        else
            ReportBug.InputBox.Reason = PTR_IssueReporter.Data.SELF_REPORTED_CONFUSED
            ReportBug.InputBox.Title:SetText("Confused Report")
            ReportBug.InputBox:Show()
            ReportBug.InputBox.InputField:SetFocus()
        end
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
    end)


    Confused:HookScript("OnEnter", function() PTR_IssueReporter.Reminder(false, Confused, ReportBug) end)
    ReportBug:HookScript("OnEnter", function() PTR_IssueReporter.Reminder(false, Confused, ReportBug) end)

    ReportBug:SetPoint("TOPLEFT", PTR_IssueReporter.Body, "TOP", 2, -6)
    Confused:SetPoint("TOPRIGHT", PTR_IssueReporter.Body, "TOP", -2, -6)
    PTR_IssueReporter.Body.Texture = PTR_IssueReporter.Body:CreateTexture()
    PTR_IssueReporter.Body.Texture:SetTexture(PTR_IssueReporter.Data.textureFile)
    PTR_IssueReporter.Body.Texture:SetPoint("TOPLEFT", Confused, "TOPLEFT")
    PTR_IssueReporter.Body.Texture:SetPoint("BOTTOMRIGHT", ReportBug, "BOTTOMRIGHT")
    PTR_IssueReporter.Body.Texture:SetDrawLayer("BACKGROUND")

    --behaviors
    FramePainter.AddDrag(PTR_IssueReporter)
    PTR_IssueReporter:SetScript("OnDragStop", function(self)
        self:StopMovingOrSizing()
        local left, bottom, width, height = self:GetRect()
        Blizzard_PTRIssueReporter_Saved.x = left
        Blizzard_PTRIssueReporter_Saved.y = bottom
    end)
    PTR_IssueReporter:SetScript("OnHide", function(self)
        self:Show()
    end)

    --timers/reminders/notifications, only one time ever on login
    C_Timer.After(1, function(self) PTR_IssueReporter.Reminder(true, Confused, ReportBug) end)

    --------------------------------------------------ALWAYS DISPLAYED BUTTONS--------------------------------------------------
    --========================================================================================================================--

    --========================================================================================================================--
    ------------------------------------------------BOSS KILL / RARE KILL PROMPT------------------------------------------------

    PTR_IssueReporter.AlertFrame = CreateFrame("Frame", "PTRIssueReporterAlertFrame", UIParent)
    PTR_IssueReporter.AlertFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 16, -116)
    PTR_IssueReporter.AlertFrame:SetSize(338, 496)
    PTR_IssueReporter.AlertFrame.TitleBox = CreateFrame("Frame", nil, PTR_IssueReporter.AlertFrame)
    PTR_IssueReporter.AlertFrame.TitleBox:SetPoint("BOTTOM", PTR_IssueReporter.AlertFrame, "TOP", 0, 2)
    PTR_IssueReporter.AlertFrame.TitleBox.text = PTR_IssueReporter.AlertFrame.TitleBox:CreateFontString("CheckListText", "OVERLAY", PTR_IssueReporter.Data.fontString)
    PTR_IssueReporter.AlertFrame.TitleBox.text:SetWidth(PTR_IssueReporter.AlertFrame.TitleBox:GetWidth())
    PTR_IssueReporter.AlertFrame.TitleBox.text:SetHeight(PTR_IssueReporter.AlertFrame.TitleBox:GetHeight())
    PTR_IssueReporter.AlertFrame.TitleBox.text:SetPoint("CENTER", PTR_IssueReporter.AlertFrame.TitleBox, "CENTER", 0, 0)
    PTR_IssueReporter.AlertFrame.TitleBox.text:SetText("Public Bug Reporter")

    PTR_IssueReporter.AlertFrame.TitleBox:SetSize(PTR_IssueReporter.AlertFrame.TitleBox.text:GetWidth()*1.5, PTR_IssueReporter.AlertFrame.TitleBox.text:GetHeight()*2)
    FramePainter.AddBackground(PTR_IssueReporter.AlertFrame.TitleBox, PTR_IssueReporter.Data.textureFile2)
    FramePainter.AddBorder(PTR_IssueReporter.AlertFrame.TitleBox)
    FramePainter.AddBorder(PTR_IssueReporter.AlertFrame)
    FramePainter.AddBackground(PTR_IssueReporter.AlertFrame, PTR_IssueReporter.Data.watermark)
    PTR_IssueReporter.AlertFrame.ClassByID = {}

    --info button
    FramePainter.AddInfoButton(PTR_IssueReporter.AlertFrame)
    FramePainter.AddTooltip(PTR_IssueReporter.AlertFrame.InfoButton,
        "Enemy bug examples",
        "This enemy hit too hard.\nThis enemy could not be hit.\nI couldn't find this enemy.\nThis enemy never moved.",
        "ANCHOR_BOTTOMRIGHT",
        PTR_IssueReporter.AlertFrame:GetWidth() - 2*PTR_IssueReporter.AlertFrame.InfoButton:GetWidth())
    --check buttons
    PTR_IssueReporter.AlertFrame.CheckButtons = {}
    --queue list
    PTR_IssueReporter.AlertFrame.Queue = {}
    PTR_IssueReporter.AlertFrame.Classification = {}
    PTR_IssueReporter.AlertFrame.Defeated = {}
    PTR_IssueReporter.AlertFrame.Choice = 0
    --model frame
    PTR_IssueReporter.AlertFrame.CreatureID = 12435
    PTR_IssueReporter.AlertFrame.Name = "Razorgore the Untamed"
    PTR_IssueReporter.AlertFrame.Model = CreateFrame("PlayerModel", nil, PTR_IssueReporter.AlertFrame)
    PTR_IssueReporter.AlertFrame.Model:SetPoint("CENTER")
    PTR_IssueReporter.AlertFrame.Model:SetSize(PTR_IssueReporter.AlertFrame:GetWidth()*0.9, PTR_IssueReporter.AlertFrame:GetHeight()*(2/3))
    PTR_IssueReporter.AlertFrame.Model:SetCreature(PTR_IssueReporter.AlertFrame.CreatureID)
    --model/name/location
    PTR_IssueReporter.AlertFrame.text = CreateFrame("Button", nil, PTR_IssueReporter.AlertFrame, "UIPanelButtonTemplate")
    PTR_IssueReporter.AlertFrame.text:SetButtonState("NORMAL", true)
    PTR_IssueReporter.AlertFrame.text:SetNormalTexture(PTR_IssueReporter.Data.textureFile2)
    PTR_IssueReporter.AlertFrame.text:EnableMouse(false)
    PTR_IssueReporter.AlertFrame.text:SetPoint("TOP", PTR_IssueReporter.AlertFrame, "TOP", 0, -15)
    PTR_IssueReporter.AlertFrame.text:SetText(PTR_IssueReporter.Data.npcAlertQuestion)
    PTR_IssueReporter.AlertFrame.text:SetSize(math.min(PTR_IssueReporter.AlertFrame.text:GetTextWidth()*1.5, PTR_IssueReporter.AlertFrame:GetWidth()*0.9), 40)
    FramePainter.AddBorder(PTR_IssueReporter.AlertFrame.text)
    --close button
    PTR_IssueReporter.AlertFrame.CloseButton = CreateFrame("Button", nil, PTR_IssueReporter.AlertFrame, "UIPanelCloseButton")
    PTR_IssueReporter.AlertFrame.CloseButton:SetPoint("CENTER", PTR_IssueReporter.AlertFrame, "TOPRIGHT")
    PTR_IssueReporter.AlertFrame.CloseButton:SetScript("OnHide", function(self)
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
    end)
    table.insert(UISpecialFrames, PTR_IssueReporter.AlertFrame:GetName())
    --text
    PTR_IssueReporter.AlertFrame.Title = PTR_IssueReporter.AlertFrame:CreateFontString("CheckListText", "OVERLAY", PTR_IssueReporter.Data.fontString)
    PTR_IssueReporter.AlertFrame.Title:SetText(PTR_IssueReporter.AlertFrame.Name)
    PTR_IssueReporter.AlertFrame.Title:SetSize(PTR_IssueReporter.AlertFrame.Title:GetStringWidth(), PTR_IssueReporter.AlertFrame.Title:GetStringHeight()*2)
    PTR_IssueReporter.AlertFrame.Title:SetPoint("TOP", PTR_IssueReporter.AlertFrame.text, "BOTTOM", 0, -8)

    PTR_IssueReporter.AlertFrame:SetScript("OnUpdate", function(self, elapsed)
        PTR_IssueReporter.AlertFrame.Timer = (PTR_IssueReporter.AlertFrame.Timer or 0) + elapsed
        if (PTR_IssueReporter.AlertFrame.Timer > math.pi*2) then
            PTR_IssueReporter.AlertFrame.Timer = 0
        end
        PTR_IssueReporter.AlertFrame.Model:SetRotation(PTR_IssueReporter.AlertFrame.Timer)
    end)

    PTR_IssueReporter.AlertFrame.SubmitButton = CreateFrame("Button", nil, PTR_IssueReporter.AlertFrame, "UIPanelButtonTemplate")
    PTR_IssueReporter.AlertFrame.SubmitButton:SetPoint("TOP", PTR_IssueReporter.AlertFrame, "BOTTOM")
    PTR_IssueReporter.AlertFrame.SubmitButton:SetText(PTR_IssueReporter.Data.SubmitText)
    PTR_IssueReporter.AlertFrame.SubmitButton:SetSize(PTR_IssueReporter.AlertFrame.SubmitButton:GetTextWidth()*1.5, 32)
    
    PTR_IssueReporter.AlertFrame:Hide()

    function PTR_IssueReporter.SetBossInfo(name, creatureId, difficulty)
        PTR_IssueReporter.AlertFrame.CreatureID = creatureId
        PTR_IssueReporter.AlertFrame.Name = name
        PTR_IssueReporter.AlertFrame.Difficulty = difficulty

        PTR_IssueReporter.AlertFrame.Model:SetCreature(PTR_IssueReporter.AlertFrame.CreatureID)
        if (#PTR_IssueReporter.AlertFrame.Difficulty == 0) then
            PTR_IssueReporter.AlertFrame.Title:SetText(string.format("%s", PTR_IssueReporter.AlertFrame.Name))
        else
            PTR_IssueReporter.AlertFrame.Title:SetText(string.format("(%s)\n%s", PTR_IssueReporter.AlertFrame.Difficulty, PTR_IssueReporter.AlertFrame.Name))
        end
        PTR_IssueReporter.AlertFrame.Title:SetWidth(math.min(PTR_IssueReporter.AlertFrame.Title:GetStringWidth()*1.5, PTR_IssueReporter.AlertFrame:GetWidth()*0.9))
        PTR_IssueReporter.AlertFrame.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.AlertFrame.CheckButtons, 0)
        PTR_IssueReporter.AlertFrame.Model:SetAnimation(190)
    end

    function PTR_IssueReporter.PushNextBoss()
        --in combat fail
        if (UnitAffectingCombat(PTR_IssueReporter.Data.unitToken)) then
            return
        end
        if (#PTR_IssueReporter.AlertFrame.Queue <= 0) then
            --no bosses to push
            PTR_IssueReporter.AlertFrame:Hide()
        else
            --take the first boss and set it if not flagged
            PTR_IssueReporter.SetBossInfo(PTR_IssueReporter.AlertFrame.Queue[#PTR_IssueReporter.AlertFrame.Queue].name,
                PTR_IssueReporter.AlertFrame.Queue[#PTR_IssueReporter.AlertFrame.Queue].id,
                PTR_IssueReporter.AlertFrame.Queue[#PTR_IssueReporter.AlertFrame.Queue].difficulty)
            PTR_IssueReporter.Data.alertFrameText = PTR_IssueReporter.Data.npcAlertQuestion
            PTR_IssueReporter.AlertFrame:Show()
        end
    end

    function PTR_IssueReporter.FlagDefeated() --if a user has submitted feedback on a boss already, don't ask again
        local lastBoss = PTR_IssueReporter.AlertFrame.Name
        local difficultyID = PTR_IssueReporter.AlertFrame.Difficulty
        PTR_IssueReporter.AlertFrame.Defeated[difficultyID] = PTR_IssueReporter.AlertFrame.Defeated[difficultyID] or {}
        PTR_IssueReporter.AlertFrame.Defeated[difficultyID][lastBoss] = true
    end

    function PTR_IssueReporter.ClearNextBoss()
        PTR_IssueReporter.AlertFrame.Queue[#PTR_IssueReporter.AlertFrame.Queue] = nil
        --also clean up any flagged bosses here
        for i=#PTR_IssueReporter.AlertFrame.Queue,1,-1 do
            local isDefeated = PTR_IssueReporter.AlertFrame.Defeated[PTR_IssueReporter.AlertFrame.Queue[i].difficulty] or {}
            if (isDefeated[PTR_IssueReporter.AlertFrame.Queue[i].name]) then
                PTR_IssueReporter.AlertFrame.Queue[i] = nil
            end
        end
    end

    function PTR_IssueReporter.AddBoss(name, creatureId, difficultyName)
        local roll = math.random(0, 99)
        if (roll < PTR_IssueReporter.Data.bossKillProbabilityPopup) then
            if (#PTR_IssueReporter.AlertFrame.Queue >= PTR_IssueReporter.Data.AlertQueueSize) then
                --push out the first one
                local numAhead = 1 + (#PTR_IssueReporter.AlertFrame.Queue - PTR_IssueReporter.Data.AlertQueueSize)
                local numIterations = PTR_IssueReporter.Data.AlertQueueSize - 1
                for i = 1, numIterations do
                    PTR_IssueReporter.AlertFrame.Queue[i] = PTR_IssueReporter.AlertFrame.Queue[i+numAhead]
                end
                for i = #PTR_IssueReporter.AlertFrame.Queue, numIterations+1, -1 do
                    PTR_IssueReporter.AlertFrame.Queue[i] = nil
                end
            end
            table.insert(PTR_IssueReporter.AlertFrame.Queue, {name = name, id = creatureId, difficulty = difficultyName})
        end
    end
    
    function PTR_IssueReporter.CheckValidLocation(searchedArray, currentLocation)
        for index, value in ipairs(searchedArray) do
            if value == currentLocation then
                return false
            end
        end
        return true
    end

    do
        PTR_IssueReporter.AlertFrame.CheckButtons[1] = FramePainter.NewCheckBox("CENTER", PTR_IssueReporter.AlertFrame, "BOTTOMLEFT", "Yes", PTR_IssueReporter.AlertFrame:GetWidth()*(1/3), 80)
        PTR_IssueReporter.AlertFrame.CheckButtons[2] = FramePainter.NewCheckBox("CENTER", PTR_IssueReporter.AlertFrame, "BOTTOMLEFT", "No", PTR_IssueReporter.AlertFrame:GetWidth()*(2/3), 80)

        PTR_IssueReporter.AlertFrame.AdditionalInfo = FramePainter.NewEditBox("BugAdditionalCommentsBossKill", "BOTTOM", PTR_IssueReporter.AlertFrame, "BOTTOM", "Additional Comments?", PTR_IssueReporter.AlertFrame:GetWidth(), 40)
        FramePainter.AddBackground(PTR_IssueReporter.AlertFrame.AdditionalInfo, PTR_IssueReporter.Data.textureFile)
        --set scripts
        for i=1,#PTR_IssueReporter.AlertFrame.CheckButtons do
            PTR_IssueReporter.AlertFrame.CheckButtons[i]:SetScript("OnClick", function(self,button,down)
                PTR_IssueReporter.AlertFrame.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.AlertFrame.CheckButtons, i)
                PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
            end)
        end
    end

    PTR_IssueReporter.AlertFrame.SubmitButton:SetScript("OnClick", function(self, button, down)
        --close and submit
        if (PTR_IssueReporter.AlertFrame.Difficulty) then
            local commentField = PTR_IssueReporter.AlertFrame.AdditionalInfo:GetText()
            commentField = commentField:gsub(",", " ")
            PTR_IssueReporter.SendResults(PTR_IssueReporter.Data.BOSS_KILL, PTR_IssueReporter.AlertFrame.Choice, PTR_IssueReporter.AlertFrame.Name, PTR_IssueReporter.AlertFrame.CreatureID, PTR_IssueReporter.AlertFrame.Difficulty, commentField)
            --submit answers
            PTR_IssueReporter.FlagDefeated()
            PTR_IssueReporter.ClearNextBoss()
            PTR_IssueReporter.PushNextBoss()
            UIErrorsFrame:AddMessage(PTR_IssueReporter.Data.Thanks)
            PTR_IssueReporter.AlertFrame.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.AlertFrame.CheckButtons, 0)
            PTR_IssueReporter.AlertFrame.AdditionalInfo:SetText("")
        else
            PTR_IssueReporter.AlertFrame:Hide()
        end
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
    end)

    PTR_IssueReporter.AlertFrame:SetScript("OnShow", function(self)
        --init data
        PTR_IssueReporter.AlertFrame.text:SetText(PTR_IssueReporter.Data.alertFrameText)
        PTR_IssueReporter.AlertFrame.AdditionalInfo:SetText("")
        PTR_IssueReporter.AlertFrame.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.AlertFrame.CheckButtons, 0)
        PlaySound(620)
    end)

    PTR_IssueReporter.AlertFrame:SetScript("OnEvent", function(self, event, ...)
        if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
            local eventArgs = {...}
            if (CombatLogGetCurrentEventInfo) then
                eventArgs = {CombatLogGetCurrentEventInfo()}
            end
            local combatevent = eventArgs[2]
            if (combatevent == "PARTY_KILL") or (UnitControllingVehicle(PTR_IssueReporter.Data.unitToken) and combatevent == "UNIT_DIED") then
                local creatureName = eventArgs[9]
                local creatureID = eventArgs[8]
                local unittype,_,_,_,_,id,guid = strsplit("-",creatureID,7)
                local instanceName, instanceType, difficultyID, difficultyName, _ = GetInstanceInfo()
                --key by difficulty
                PTR_IssueReporter.AlertFrame.Classification[difficultyName] = PTR_IssueReporter.AlertFrame.Classification[difficultyName] or {}
                PTR_IssueReporter.AlertFrame.Defeated[difficultyName] = PTR_IssueReporter.AlertFrame.Defeated[difficultyName] or {}
                if (PTR_IssueReporter.AlertFrame.Classification[difficultyName][creatureName]) then
                    if (not PTR_IssueReporter.AlertFrame.Defeated[difficultyName][creatureName]) then
                        if (PTR_IssueReporter.AlertFrame.ClassByID[id]) and (PTR_IssueReporter.AlertFrame.ClassByID[id] == "rare" or PTR_IssueReporter.AlertFrame.ClassByID[id] == "worldboss" or PTR_IssueReporter.AlertFrame.ClassByID[id] == "rareelite") then
                            if (PTR_IssueReporter.CheckValidLocation(IgnoreLocations, select(8,GetInstanceInfo()))) then
                                PTR_IssueReporter.AddBoss(creatureName, id, difficultyName)
                            end
                        end
                    end
                end
            end
        elseif (event == "ENCOUNTER_END") then
            local bossname = select(2,...)
            local instanceName, instanceType, difficultyID, difficultyName, _ = GetInstanceInfo()
            PTR_IssueReporter.AlertFrame.Classification[difficultyName] = PTR_IssueReporter.AlertFrame.Classification[difficultyName] or {}
            local bossID = PTR_IssueReporter.AlertFrame.Classification[difficultyName][bossname]
            if (bossID) then
                PTR_IssueReporter.AddBoss(bossname, bossID, difficultyName)
            end
        elseif (event == "UNIT_TARGET") then
            if (select(1,...) == PTR_IssueReporter.Data.unitToken) then
                local classid = UnitClassification(PTR_IssueReporter.Data.targetToken)
                local name = UnitName(PTR_IssueReporter.Data.targetToken)
                local _,_,_,_,_,creatureID,guid = strsplit("-", UnitGUID(PTR_IssueReporter.Data.targetToken) or "", 7)
                local instanceName, instanceType, difficultyID, difficultyName, _ = GetInstanceInfo()
                PTR_IssueReporter.AlertFrame.Classification[difficultyName] = PTR_IssueReporter.AlertFrame.Classification[difficultyName] or {}
                if (name and classid) then
                    if (classid == "elite" or classid == "worldboss" or classid == "rare" or classid == "rareelite") and (not PTR_IssueReporter.AlertFrame.Classification[difficultyName][name]) then
                        PTR_IssueReporter.AlertFrame.Classification[difficultyName][name] = creatureID
                        PTR_IssueReporter.AlertFrame.ClassByID[creatureID] = classid
                    end
                end
            end
        elseif (event == "PLAYER_REGEN_ENABLED") then
            --combat ended
            PTR_IssueReporter.PushNextBoss()
        elseif (event == "PLAYER_REGEN_DISABLED") then
            --combat began
            PTR_IssueReporter.AlertFrame:Hide()
        end
    end)
    PTR_IssueReporter.AlertFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    PTR_IssueReporter.AlertFrame:RegisterEvent("UNIT_TARGET")
    PTR_IssueReporter.AlertFrame:RegisterEvent("ENCOUNTER_END")
    PTR_IssueReporter.AlertFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
    PTR_IssueReporter.AlertFrame:RegisterEvent("PLAYER_REGEN_DISABLED")

    ------------------------------------------------BOSS KILL / RARE KILL PROMPT------------------------------------------------
    --========================================================================================================================--

    --========================================================================================================================--
    ---------------------------------------------------IMPORTANT QUEST PROMPT---------------------------------------------------

    PTR_IssueReporter.QuestSubmit = CreateFrame("Frame", nil, QuestFrame)
    PTR_IssueReporter.QuestSubmit:Hide()
    PTR_IssueReporter.QuestSubmit:SetSize(QuestFrame:GetWidth() - 4, 150)
    FramePainter.AddBackground(PTR_IssueReporter.QuestSubmit, PTR_IssueReporter.Data.watermark)
    FramePainter.AddBorder(PTR_IssueReporter.QuestSubmit)
    PTR_IssueReporter.QuestSubmit:SetPoint("TOPLEFT", QuestFrame, "BOTTOMLEFT")
    PTR_IssueReporter.QuestSubmit.CheckButtons = {}
    PTR_IssueReporter.QuestSubmit.Choice = 0
    PTR_IssueReporter.QuestSubmit.History = {}

    --question
    PTR_IssueReporter.QuestSubmit.text = PTR_IssueReporter.QuestSubmit:CreateFontString("CheckListText", "OVERLAY", PTR_IssueReporter.Data.fontString)

    PTR_IssueReporter.QuestSubmit.text:SetText("|cffFFFFFFPublic Bug Reporter|r\nDid you experience any bugs or problems\nwith this quest?")

    PTR_IssueReporter.QuestSubmit.text:SetSize(PTR_IssueReporter.QuestSubmit:GetWidth(), PTR_IssueReporter.QuestSubmit.text:GetHeight())
    PTR_IssueReporter.QuestSubmit.text:SetPoint("TOP", PTR_IssueReporter.QuestSubmit, "TOP", 0, -8)

    FramePainter.AddInfoButton(PTR_IssueReporter.QuestSubmit, "TOPRIGHT")
    FramePainter.AddTooltip(PTR_IssueReporter.QuestSubmit.InfoButton,
        "Quest bug examples",
        "This quest was hard to find.\nThe marker on my map wasn't where I needed to go.\nIt took a long time to complete this quest.",
        "ANCHOR_BOTTOMRIGHT")

    function PTR_IssueReporter.IsTargetQuest()
        if (#PTR_IssueReporter.Data.targetQuests < 1) then
            return true
        else
            local currentQuestID = GetQuestID() or 0
            for k,v in pairs(PTR_IssueReporter.Data.targetQuests) do
                if (currentQuestID == v) then
                    return true
                end
            end
        end

        return false
    end

    function PTR_IssueReporter.AddQuestHistory(id)
        table.insert(PTR_IssueReporter.QuestSubmit.History, id, 1)
        local maxSize = PTR_IssueReporter.Data.QuestHistorySize
        if (#PTR_IssueReporter.QuestSubmit.History > maxSize) then
            for i = #PTR_IssueReporter.QuestSubmit.History,(maxSize+1),-1 do
                PTR_IssueReporter.QuestSubmit.History[i] = nil
            end
        end
    end

    function PTR_IssueReporter.GetQuestHistory()
        local historyString = ""
        local firstQuest = true
        for k,v in ipairs(PTR_IssueReporter.QuestSubmit.History) do
            if (firstQuest) then
                historyString = string.format("%s", v)
                firstQuest = false
            else
                historyString = string.format("%s:%s", historyString, v)
            end
        end

        return historyString
    end

    --add checkboxes on the popup
    do
        --input field for more comments
        PTR_IssueReporter.QuestSubmit.AdditionalInfo = FramePainter.NewEditBox("IssueReporterAdditionalComments", "BOTTOM", PTR_IssueReporter.QuestSubmit, "BOTTOM", "Additional Comments?", PTR_IssueReporter.QuestSubmit:GetWidth(), 40)
        FramePainter.AddBackground(PTR_IssueReporter.QuestSubmit.AdditionalInfo, PTR_IssueReporter.Data.textureFile)
        PTR_IssueReporter.QuestSubmit.CheckButtons[1] = FramePainter.NewCheckBox("BOTTOMRIGHT", PTR_IssueReporter.QuestSubmit.AdditionalInfo, "TOP", "Yes", -20, 25, 1, 1, 1)
        PTR_IssueReporter.QuestSubmit.CheckButtons[2] = FramePainter.NewCheckBox("BOTTOMLEFT", PTR_IssueReporter.QuestSubmit.AdditionalInfo, "TOP", "No", 20, 25, 1, 1, 1)
        --set scripts
        for i=1,#PTR_IssueReporter.QuestSubmit.CheckButtons do
            PTR_IssueReporter.QuestSubmit.CheckButtons[i]:SetScript("OnClick", function(self,button,down)
                PTR_IssueReporter.QuestSubmit.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.QuestSubmit.CheckButtons, i)
                PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
            end)
        end
    end

    PTR_IssueReporter.QuestSubmit:SetScript("OnShow", function(self)
        --empty comment box
        PTR_IssueReporter.QuestSubmit.AdditionalInfo:SetText("")
        --reset checkbox
        PTR_IssueReporter.QuestSubmit.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.QuestSubmit.CheckButtons, 0)
    end)
    --listen for QUEST_DETAIL
    PTR_IssueReporter.QuestSubmit:SetScript("OnEvent", function(self, event, ...)
        if (event == "QUEST_COMPLETE") then
            if (PTR_IssueReporter.IsTargetQuest() and (not GetCVarBool("showNPETutorials")) or (UnitLevel(PTR_IssueReporter.Data.unitToken) >= (NPE_TUTORIAL_COMPLETE_LEVEL or 1))) then
                self:Show()
            end
        elseif (event == "QUEST_TURNED_IN") then
            if (not GetCVarBool("showNPETutorials")) or (UnitLevel(PTR_IssueReporter.Data.unitToken) >= (NPE_TUTORIAL_COMPLETE_LEVEL or 1)) then
                local questID = select(1,...)
                --submit diagnostics
                local commentField = PTR_IssueReporter.QuestSubmit.AdditionalInfo:GetText()
                commentField = commentField:gsub(",", "")
                if (PTR_IssueReporter.QuestSubmit.Choice > 0) or (#commentField > 0) then
                    PTR_IssueReporter.SendResults(PTR_IssueReporter.Data.QUEST_TURNED_IN, PTR_IssueReporter.QuestSubmit.Choice, questID, commentField, PTR_IssueReporter.GetQuestHistory())
                    UIErrorsFrame:AddMessage(PTR_IssueReporter.Data.Thanks)
                end
                PTR_IssueReporter.QuestSubmit.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.QuestSubmit.CheckButtons, 0)
                PTR_IssueReporter.QuestSubmit.AdditionalInfo:SetText("")
            end
            self:Hide()
        elseif (event == "QUEST_FINISHED") then
            PTR_IssueReporter.QuestSubmit.Choice = PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.QuestSubmit.CheckButtons, 0)
            PTR_IssueReporter.QuestSubmit.AdditionalInfo:SetText("")
            self:Hide()
        end
    end)

    PTR_IssueReporter.QuestSubmit:RegisterEvent("QUEST_COMPLETE")
    PTR_IssueReporter.QuestSubmit:RegisterEvent("QUEST_TURNED_IN")
    PTR_IssueReporter.QuestSubmit:RegisterEvent("QUEST_FINISHED") --called when quest window closes

---------------------------------------------------IMPORTANT QUEST PROMPT---------------------------------------------------
--========================================================================================================================--
    PTR_IssueReporter.EventPopup = CreateFrame("Frame", nil, UIParent)
    PTR_IssueReporter.EventPopup.Reason = 0
    PTR_IssueReporter.EventPopup:SetSize(400, 30)
    PTR_IssueReporter.EventPopup:SetPoint("LEFT", UIParent, "LEFT", 50, UIParent:GetHeight()*0.33)
    FramePainter.AddBackground(PTR_IssueReporter.EventPopup, PTR_IssueReporter.Data.textureFile2)
    FramePainter.AddBorder(PTR_IssueReporter.EventPopup)
    PTR_IssueReporter.EventPopup:SetMovable(true)
    PTR_IssueReporter.EventPopup:EnableMouse(true)
    PTR_IssueReporter.EventPopup:SetClampedToScreen(true)

    PTR_IssueReporter.EventPopup:RegisterForDrag("LeftButton")
    PTR_IssueReporter.EventPopup:SetScript("OnDragStart", function(self)
        self:StartMoving()
    end)
    PTR_IssueReporter.EventPopup:SetScript("OnDragStop", function(self)
        self:StopMovingOrSizing()
    end)

    PTR_IssueReporter.EventPopup.Label = PTR_IssueReporter.EventPopup:CreateFontString(nil, "OVERLAY")
    PTR_IssueReporter.EventPopup.Label:SetFont("Fonts\\FRIZQT__.TTF", PTR_IssueReporter.EventPopup:GetHeight()*0.5, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.Label:SetPoint("LEFT", PTR_IssueReporter.EventPopup, "LEFT", 8, 0)
    PTR_IssueReporter.EventPopup.Label:SetJustifyH("LEFT")
    PTR_IssueReporter.EventPopup.Label:SetTextColor(1, 1, 1, 1)
    PTR_IssueReporter.EventPopup.TopLabel = PTR_IssueReporter.EventPopup:CreateFontString(nil, "OVERLAY")
    PTR_IssueReporter.EventPopup.TopLabel:SetFont("Fonts\\FRIZQT__.TTF", PTR_IssueReporter.EventPopup:GetHeight()*0.8, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.TopLabel:SetText("Bug Reporter")
    PTR_IssueReporter.EventPopup.TopLabel:SetTextColor(1, 0.8, 0.2, 1)
    PTR_IssueReporter.EventPopup.TopLabel:SetPoint("BOTTOM", PTR_IssueReporter.EventPopup, "TOP")

    PTR_IssueReporter.EventPopup.CloseButton = CreateFrame("Button", nil, PTR_IssueReporter.EventPopup, "UIPanelCloseButtonNoScripts")
    PTR_IssueReporter.EventPopup.CloseButton:SetPoint("RIGHT", PTR_IssueReporter.EventPopup, "RIGHT")
    PTR_IssueReporter.EventPopup.CloseButton:SetScript("OnClick", function(self)
        PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
        PTR_IssueReporter.EventPopup:Hide()
    end)
    PTR_IssueReporter.EventPopup.Label:SetPoint("RIGHT", PTR_IssueReporter.EventPopup.CloseButton, "LEFT", 0, 0)

    PTR_IssueReporter.EventPopup.Pool = {}
    PTR_IssueReporter.EventPopup.CheckBoxes = CreateFrame("Frame", nil, PTR_IssueReporter.EventPopup)
    PTR_IssueReporter.EventPopup.CheckBoxes:SetPoint("TOPLEFT", PTR_IssueReporter.EventPopup, "BOTTOMLEFT")
    PTR_IssueReporter.EventPopup.CheckBoxes:SetPoint("TOPRIGHT", PTR_IssueReporter.EventPopup, "BOTTOMRIGHT")
    FramePainter.AddBackground(PTR_IssueReporter.EventPopup.CheckBoxes, PTR_IssueReporter.Data.textureFile2)
    FramePainter.AddBorder(PTR_IssueReporter.EventPopup.CheckBoxes)
    PTR_IssueReporter.EventPopup.CheckBoxes:SetHeight(100)
    PTR_IssueReporter.EventPopup.CheckBoxes.Question = PTR_IssueReporter.EventPopup.CheckBoxes:CreateFontString(nil, "OVERLAY")
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetPoint("TOPLEFT", PTR_IssueReporter.EventPopup.CheckBoxes, "TOPLEFT", 8, -8)
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetPoint("TOPRIGHT", PTR_IssueReporter.EventPopup.CheckBoxes, "TOPRIGHT", -8, -8)
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetJustifyV("TOP")
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetJustifyH("LEFT")
    PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetTextColor(1, 0.8, 0.2, 1)
    PTR_IssueReporter.EventPopup.CheckBoxes:SetHeight(PTR_IssueReporter.EventPopup.CheckBoxes.Question:GetHeight() + 15)

    PTR_IssueReporter.EventPopup.Body = CreateFrame("Frame", nil, PTR_IssueReporter.EventPopup)
    PTR_IssueReporter.EventPopup.Body:SetHeight(200)
    PTR_IssueReporter.EventPopup.Body:EnableMouse(true)
    PTR_IssueReporter.EventPopup.Body:SetPoint("TOPLEFT", PTR_IssueReporter.EventPopup.CheckBoxes, "BOTTOMLEFT")
    PTR_IssueReporter.EventPopup.Body:SetPoint("TOPRIGHT", PTR_IssueReporter.EventPopup.CheckBoxes, "BOTTOMRIGHT")
    FramePainter.AddBackground(PTR_IssueReporter.EventPopup.Body, PTR_IssueReporter.Data.watermark)
    FramePainter.AddBorder(PTR_IssueReporter.EventPopup.Body)
    PTR_IssueReporter.EventPopup.Body.Counter = PTR_IssueReporter.EventPopup.Body:CreateFontString(nil, "OVERLAY")
    PTR_IssueReporter.EventPopup.Body.Counter:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.Body.Counter:SetPoint("BOTTOMRIGHT", PTR_IssueReporter.EventPopup.Body, "BOTTOMRIGHT")

    PTR_IssueReporter.EventPopup.Body.EditBox = CreateFrame("EditBox", nil, PTR_IssueReporter.EventPopup.Body)
    PTR_IssueReporter.EventPopup.Body.EditBox.MaxLetters = 255
    PTR_IssueReporter.EventPopup.Body.EditBox:SetAllPoints(PTR_IssueReporter.EventPopup.Body)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetFont("Fonts\\FRIZQT__.TTF", 16, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.Body.EditBox:SetAutoFocus(false)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetTextInsets(12, 12, 8, -8)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetJustifyV("TOP")
    PTR_IssueReporter.EventPopup.Body.EditBox:SetJustifyH("LEFT")
    PTR_IssueReporter.EventPopup.Body.EditBox:SetMultiLine(true)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetMaxLetters(PTR_IssueReporter.EventPopup.Body.EditBox.MaxLetters)
    PTR_IssueReporter.EventPopup.Body.EditBox.Preface = PTR_IssueReporter.EventPopup.Body.EditBox:CreateFontString(nil, "OVERLAY")
    PTR_IssueReporter.EventPopup.Body.EditBox.Preface:SetFont("Fonts\\FRIZQT__.TTF", 16, "OUTLINE, THICK")
    PTR_IssueReporter.EventPopup.Body.EditBox.Preface:SetPoint("TOPLEFT", PTR_IssueReporter.EventPopup.Body, "TOPLEFT", 12, -8)
    PTR_IssueReporter.EventPopup.Body.EditBox.Preface:SetText("Additional Comments?")
    PTR_IssueReporter.EventPopup.Body.EditBox.Preface:SetTextColor(1, 1, 1, 0.5)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetScript("OnEnterPressed", function(self)
        self:ClearFocus()
    end)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetScript("OnEscapePressed", function(self)
        self:ClearFocus()
    end)
    PTR_IssueReporter.EventPopup.Body.EditBox:SetScript("OnTextChanged", function(self)
        local body = self:GetText()
        if (#body > 0) then
            PTR_IssueReporter.EventPopup.Body.EditBox.Preface:Hide()
        else
            PTR_IssueReporter.EventPopup.Body.EditBox.Preface:Show()
        end
        local count = string.format("%s / %s", #body, PTR_IssueReporter.EventPopup.Body.EditBox.MaxLetters)
        PTR_IssueReporter.EventPopup.Body.Counter:SetText(count)
    end)

    PTR_IssueReporter.EventPopup.Footer = CreateFrame("Frame", nil, PTR_IssueReporter.EventPopup)
    PTR_IssueReporter.EventPopup.Footer:SetHeight(35)
    PTR_IssueReporter.EventPopup.Footer:SetPoint("TOPLEFT", PTR_IssueReporter.EventPopup.Body, "BOTTOMLEFT", 0, 0)
    PTR_IssueReporter.EventPopup.Footer:SetPoint("TOPRIGHT", PTR_IssueReporter.EventPopup.Body, "BOTTOMRIGHT", 0, 0)

    PTR_IssueReporter.EventPopup.SubmitButton = CreateFrame("Button", nil, PTR_IssueReporter.EventPopup.Footer, "UIPanelButtonTemplate")
    PTR_IssueReporter.EventPopup.SubmitButton:SetSize(140, 35)
    PTR_IssueReporter.EventPopup.SubmitButton:SetPoint("TOP", PTR_IssueReporter.EventPopup.Footer, "TOP")
    PTR_IssueReporter.EventPopup.SubmitButton:SetText(PTR_IssueReporter.Data.SubmitText)
    PTR_IssueReporter.EventPopup.SubmitButton:SetScript("OnClick", function(self, button, down)
        if (button == "LeftButton") then
            PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
            PTR_IssueReporter.EventPopup:Hide()

            local choice, reason, comments, refID = PTR_IssueReporter.GetEventInfo()
            if (choice > 0) and (comments) and (#comments > 0) then
                PTR_IssueReporter.SendResults(reason, choice, refID, comments)
            end
            PTR_IssueReporter.EventPopup.Reason = 0
            UIErrorsFrame:AddMessage(PTR_IssueReporter.Data.Thanks)
        end
    end)
    
    PTR_IssueReporter.EventPopup:SetClampRectInsets(0, 0, 0, -(PTR_IssueReporter.EventPopup.Body:GetHeight() + PTR_IssueReporter.EventPopup.Footer:GetHeight() + PTR_IssueReporter.EventPopup.CheckBoxes:GetHeight()))

    function PTR_IssueReporter.GetCheckBoxFromPool()
        for k,v in ipairs(PTR_IssueReporter.EventPopup.Pool) do
            if (v.isAvailable) then
                v.isAvailable = false
                return v
            end
        end

        local onDemandCheckBox = FramePainter.NewCheckBox(nil, PTR_IssueReporter.EventPopup.CheckBoxes, nil, "", 0, 0, nil, nil, nil, true)
        table.insert(PTR_IssueReporter.EventPopup.Pool, onDemandCheckBox)
        onDemandCheckBox.index = #PTR_IssueReporter.EventPopup.Pool
        onDemandCheckBox:SetScript("OnClick", function(self)
            PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
            PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.EventPopup.Pool, self.index)
        end)
        return onDemandCheckBox
    end

    function PTR_IssueReporter.SetCheckBoxes(reason)
        local eventPackage = PTR_IssueReporter.TriggerEvents[reason]
        if (not eventPackage.enabled) then
            return
        end
        local options = eventPackage.checkboxes
        local boxes = {}
        PTR_IssueReporter.ExclusiveCheckButton(PTR_IssueReporter.EventPopup.Pool, 0)
        for k,v in pairs(PTR_IssueReporter.EventPopup.Pool) do
            v.isAvailable = true
            v:Hide()
        end

        local spacing = PTR_IssueReporter.EventPopup.CheckBoxes:GetWidth() / (#options + 1)
        for k,v in ipairs(options) do
            local Box = PTR_IssueReporter.GetCheckBoxFromPool()
            Box:ClearAllPoints()
            Box.text:SetText(v)
            table.insert(boxes, Box)
            Box:SetPoint("BOTTOM", PTR_IssueReporter.EventPopup.CheckBoxes, "BOTTOMLEFT", k*spacing, 0)
            Box:Show()
        end

        PTR_IssueReporter.EventPopup.CheckBoxes:SetHeight(PTR_IssueReporter.EventPopup.CheckBoxes.Question:GetHeight() + 15 + ((#boxes > 0) and 50 or 0))
        PTR_IssueReporter.EventPopup:SetClampRectInsets(0, 0, 0, -(PTR_IssueReporter.EventPopup.Body:GetHeight() + PTR_IssueReporter.EventPopup.Footer:GetHeight() + PTR_IssueReporter.EventPopup.CheckBoxes:GetHeight()))
    end

    function PTR_IssueReporter.GetEventInfo()
        local choice, reason, comments, refID = 0, PTR_IssueReporter.EventPopup.Reason, PTR_IssueReporter.EventPopup.Body.EditBox:GetText(), 0
        if (PTR_IssueReporter.EventPopup.MapInfo) and (PTR_IssueReporter.EventPopup.MapInfo.id) then
            refID = PTR_IssueReporter.EventPopup.MapInfo.id
        end
        for k,v in ipairs(PTR_IssueReporter.EventPopup.Pool) do
            if (v:GetChecked()) then
                choice = k
                break
            end
        end

        return choice, reason, comments, refID
    end

    function PTR_IssueReporter.GetReason()
        local instanceName, instanceType, difficultyID, difficultyName, _ = GetInstanceInfo()
        local _, _, _, mapID = UnitPosition(PTR_IssueReporter.Data.unitToken)
        if (instanceName:match("[Ww]arfront")) then
            PTR_IssueReporter.EventPopup.MapInfo = {name = instanceName, id = mapID}
            return PTR_IssueReporter.Data.WARFRONTS
        end
        if (not difficultyID) then
            return 0
        end
        if (difficultyID == 45) or (difficultyID == 40) or (difficultyID == 39) or (difficultyID == 38) then
            PTR_IssueReporter.EventPopup.MapInfo = {name = instanceName, id = mapID}
            return PTR_IssueReporter.Data.ISLANDS
        end

        return 0
    end

    function PTR_IssueReporter.PopEvent(reason)
        PTR_IssueReporter.EventPopup.Reason = reason
        local eventPackage = PTR_IssueReporter.TriggerEvents[reason]
        if (not eventPackage.enabled) then
            return
        end
        local title, description = eventPackage.label, eventPackage.question
        if (PTR_IssueReporter.EventPopup.MapInfo) and (PTR_IssueReporter.EventPopup.MapInfo.name) then
            title = string.format("%s - %s", title, PTR_IssueReporter.EventPopup.MapInfo.name)
        end
        PTR_IssueReporter.EventPopup.Label:SetText(title)
        PTR_IssueReporter.EventPopup.CheckBoxes.Question:SetText(description)
        PTR_IssueReporter.EventPopup.Body.EditBox:SetText("")
        PTR_IssueReporter.EventPopup:Show()
        PlaySound(SOUNDKIT.IG_QUEST_LIST_OPEN)
    end

    PTR_IssueReporter.EventPopup:RegisterEvent("PLAYER_ENTERING_WORLD")
    PTR_IssueReporter.EventPopup:SetScript("OnEvent", function(self, event, ...)
        PTR_IssueReporter.EventPopup:Hide()
        PTR_IssueReporter.EventPopup.LastMap = PTR_IssueReporter.EventPopup.NewMap or 0
        PTR_IssueReporter.EventPopup.NewMap = PTR_IssueReporter.GetReason()

        if (PTR_IssueReporter.EventPopup.LastMap > 0) and (PTR_IssueReporter.EventPopup.NewMap == 0) then
            C_Timer.After(3, function()
                PTR_IssueReporter.PopEvent(PTR_IssueReporter.EventPopup.LastMap)
                PTR_IssueReporter.SetCheckBoxes(PTR_IssueReporter.EventPopup.LastMap)
            end)
        end
    end)
    PTR_IssueReporter.EventPopup:Hide()
    PTR_IssueReporter:Show()
end

--========================================================================================================================--
if (isGMClient) then
    PTRFeedbackAddonSetup = false
    DEFAULT_CHAT_FRAME:AddMessage("[INTERNAL ONLY] Type /ptrfeedback to view options.", 1, 1, 0.5)
    SLASH_PTRFEEDBACK1 = '/ptrfeedback'
    function handler(msg)
        if (#msg == 0) then
            DEFAULT_CHAT_FRAME:AddMessage("PTR Feedback is currently disabled.\nTo setup the PTR Feedback addon:\n/ptrfeedback setup", 1, 1, 0.5)
        elseif (msg:lower() == "setup") then
            if PTRFeedbackAddonSetup then
                print("The PTR Feedback Addon is already setup.")
            else
                print("Setting up the PTR Feedback Addon...")
                PTR_IssueReporter.Init()
                PTRFeedbackAddonSetup = true
            end
        end
    end
    SlashCmdList["PTRFEEDBACK"] = handler
else
    PTR_IssueReporter:RegisterEvent("ADDON_LOADED")
    PTR_IssueReporter:SetScript("OnEvent", function(self, event, ...)
        if (select(1,...) == "Blizzard_PTRFeedback") then
            PTR_IssueReporter.Init()
        end
    end)
end