--------------------------------------------------
--NOTE - Please do not change this section without understanding the full implications of the secure environment
--We usually don't want to call out of this environment from this file. Calls should usually go through Outbound
local _, tbl = ...;
tbl.SecureCapsuleGet = SecureCapsuleGet;

local function Import(name)
	tbl[name] = tbl.SecureCapsuleGet(name);
end

Import("IsOnGlueScreen");

if ( tbl.IsOnGlueScreen() ) then
	tbl._G = _G;	--Allow us to explicitly access the global environment at the glue screens
	Import("C_StoreGlue");
end

setfenv(1, tbl);

Import("pairs");
Import("select");

function Mixin(object, ...)
	for i = 1, select("#", ...) do
		local mixin = select(i, ...);
		for k, v in pairs(mixin) do
			object[k] = v;
		end
	end

	return object;
end

function CreateFromMixins(...)
	return Mixin({}, ...)
end
--------------------------------------------------

--Imports
Import("bit");
Import("C_StoreSecure");
Import("C_WowTokenPublic");
Import("math");
Import("table");
Import("ipairs");
Import("pairs");
Import("select");
Import("unpack");
Import("type");
Import("string");
Import("strtrim");
Import("PlaySound");
Import("SetPortraitToTexture");
Import("GetMouseFocus");
Import("Enum");
Import("SecureMixin");
Import("CreateFromSecureMixins");
Import("ShrinkUntilTruncateFontStringMixin");
Import("IsTrialAccount");
Import("IsVeteranTrialAccount");
Import("PortraitFrameTemplate_SetPortraitToAsset");
Import("BLIZZARD_STORE_BUY");

local BATTLEPAY_SPLASH_BANNER_TEXT_FEATURED = 0;
local BATTLEPAY_SPLASH_BANNER_TEXT_DISCOUNT = 1;
local BATTLEPAY_SPLASH_BANNER_TEXT_NEW = 2;

local SEE_YOU_LATER_BUNDLE_PRODUCT_ID = 488;


--------------------------------------------------
-- FULL STORE CARD MIXIN
FullStoreCardMixin = CreateFromMixins(StoreCardMixin);

function FullStoreCardMixin:OnLoad()
	StoreCardMixin.OnLoad(self);
	
	self.SplashBannerText:SetShadowColor(0, 0, 0, 0);

	self.ProductName:SetSpacing(0);
	self.Description:SetSpacing(2);	
end

function FullStoreCardMixin:OnEnter()
	self:ShowTooltip();
end

function FullStoreCardMixin:ShowTooltip()
	if self.productTooltipTitle then
		StoreTooltip:ClearAllPoints();
		if self.anchorRight then
			StoreTooltip:SetPoint("BOTTOMLEFT", self, "TOPRIGHT", -7, -6);
		else
			StoreTooltip:SetPoint("BOTTOMRIGHT", self, "TOPLEFT", 7, -6);
		end
		
		local entryInfo = self:GetEntryInfo();
		local description = self:AppendBundleInformationToTooltipDescription(entryInfo, self.productTooltipDescription);
		description = strtrim(description, "\n\r"); -- Ensure we don't end the description with a new line.		
		StoreTooltip_Show(self.productTooltipTitle, description);
	end
end

function FullStoreCardMixin:OnLeave()
	self.HighlightTexture:Hide();
	StoreTooltip:Hide();
end

function FullStoreCardMixin:OnClick()
	StoreProductCard_CheckShowStorePreviewOnClick(self);
end

function FullStoreCardMixin:GetCurrencyFormat(currencyInfo)
	return currencyInfo.formatLong;
end

function FullStoreCardMixin:SetupBuyButton(info, entryInfo)
	local text = BLIZZARD_STORE_BUY;
	if (info.browseBuyButtonText) then
		text = info.browseBuyButtonText;
	end
	self.BuyButton:SetText(text);
	self.BuyButton:SetEnabled(self:ShouldEnableBuyButton(entryInfo));
end

function FullStoreCardMixin:SetupBannerText(discounted, discountPercentage, entryInfo)
	if entryInfo.bannerType == BATTLEPAY_SPLASH_BANNER_TEXT_NEW then
		self.SplashBannerText:SetText(BLIZZARD_STORE_SPLASH_BANNER_NEW);
	elseif entryInfo.bannerType == BATTLEPAY_SPLASH_BANNER_TEXT_DISCOUNT then
		if discounted then
			self.SplashBannerText:SetText(string.format(BLIZZARD_STORE_SPLASH_BANNER_DISCOUNT_FORMAT, discountPercentage));
		else
			self.SplashBannerText:SetText(BLIZZARD_STORE_SPLASH_BANNER_FEATURED);
		end
	elseif entryInfo.bannerType == BATTLEPAY_SPLASH_BANNER_TEXT_FEATURED then
		self.SplashBannerText:SetText(BLIZZARD_STORE_SPLASH_BANNER_FEATURED);
	end
end

function FullStoreCardMixin:ShowDiscount(discountText)
	StoreCardMixin.ShowDiscount(self, discountText);

	local width = self.NormalPrice:GetStringWidth() + self.SalePrice:GetStringWidth();

	if (width + 120 + (self:GetWidth()/8)) > self:GetWidth() then
		self.SalePrice:ClearAllPoints();
		self.SalePrice:SetPoint("TOPLEFT", self.NormalPrice, "BOTTOMLEFT", 0, -4);
	else
		self.SalePrice:ClearAllPoints();
		self.SalePrice:SetPoint("BOTTOMLEFT", self.NormalPrice, "BOTTOMRIGHT", 4, 0);
	end
end

function FullStoreCardMixin:SetupWoWToken(entryInfo)
	if entryInfo.sharedData.productDecorator == Enum.BattlepayProductDecorator.WoWToken then
		local price = C_WowTokenPublic.GetCurrentMarketPrice();
		if (price) then
			self.CurrentMarketPrice:SetText(string.format(TOKEN_CURRENT_AUCTION_VALUE, GetSecureMoneyString(price, true)));
		else
			self.CurrentMarketPrice:SetText(string.format(TOKEN_CURRENT_AUCTION_VALUE, TOKEN_MARKET_PRICE_NOT_AVAILABLE));
		end
		self.CurrentPrice:ClearAllPoints();
		self.CurrentPrice:SetPoint("TOPLEFT", self.CurrentMarketPrice, "BOTTOMLEFT", 0, -28);
		self.NormalPrice:ClearAllPoints();
		self.NormalPrice:SetPoint("TOPLEFT", self.CurrentMarketPrice, "BOTTOMLEFT", 0, -28);
		self.CurrentMarketPrice:Show();
	else
		self.CurrentMarketPrice:Hide();
	end
end

function FullStoreCardMixin:SetupDescription(entryInfo)
	local description = entryInfo.sharedData.description;
	if entryInfo.sharedData.productDecorator == Enum.BattlepayProductDecorator.WoWToken then
		description = BLIZZARD_STORE_TOKEN_DESC_30_DAYS;
	end

	local baseDescription, bullets = description:match("(.-)$bullet(.*)");
	if not bullets then
		self.Description:SetText(description);
	else
		local bulletPoints = {};
		while bullets ~= nil and bullets ~= "" do
			local bullet = bullets:match("(.-)$bullet") or bullets;
			bullet = strtrim(bullet, "\n\r");
			bullets = bullets:match("$bullet(.*)");
			table.insert(bulletPoints, bullet);
		end

		self.Description:SetJustifyH("LEFT");

		if baseDescription ~= "" then
			self.Description:SetText(strtrim(baseDescription, "\n\r"));
			self.Description:Show();
			self.DescriptionBulletPointContainer:SetPoint("TOP", self.Description, "BOTTOM", 0, -3);
		else
			self.Description:Hide();
			self.DescriptionBulletPointContainer:SetPoint("TOP", self.Description, "TOP");
		end

		self.DescriptionBulletPointContainer:SetContents(bulletPoints);
	end
end

function FullStoreCardMixin:ShouldModelShowShadows()
	return false;
end

function FullStoreCardMixin:InitMagnifier()
	self.Magnifier:SetShown(self:ShouldShowMagnifyingGlass());
end

function FullStoreCardMixin:SetCardTexture(entryInfo)
	if entryInfo.sharedData.overrideBackground then
		self.Card:SetAtlas(entryInfo.sharedData.overrideBackground, true);
		self.Card:SetTexCoord(0, 1, 0, 1);
	else
		self.Card:SetSize(576, 471);
		self.Card:SetTexture("Interface\\Store\\Store-Main");
		self.Card:SetTexCoord(0.00097656, 0.56347656, 0.00097656, 0.46093750);
	end
end

function FullStoreCardMixin:GetTooltipOffsets()	
	local point = "RIGHT";
	local rpoint = "BOTTOMRIGHT";
	local x = self:GetLeft();
	local y = self:GetTop();
	return x, y, point, rpoint;
end

function FullStoreCardMixin:ShouldShowIcon(entryInfo)
	return StoreCardMixin.ShouldShowIcon(self, entryInfo) and entryInfo.sharedData.texture;
end

function FullStoreCardMixin:ShowIcon(displayData)
	local icon = displayData.texture or self:GetDefaultIconName();
	local shouldShow = displayData.itemID ~= nil;
	self.InvisibleMouseOverFrame:SetShown(shouldShow);
	
	self.Icon:ClearAllPoints();
	
	local overrideTexture = displayData.overrideTexture;
	local useSquareBorder = bit.band(displayData.flags, Enum.BattlepayDisplayFlag.UseSquareIconBorder) == Enum.BattlepayDisplayFlag.UseSquareIconBorder;

	if overrideTexture then
		self.IconBorder:Hide();
		self.Icon:SetAtlas(overrideTexture, true);

		local adjustX, adjustY;
		local width, height = self.Icon:GetSize();
		if (width > 64) then
			adjustX = -(width - 64);
		else
			adjustX = 64 - width;
		end

		if (height > 64) then
			adjustY = height - 64;
		else
			adjustY = -(64 - height);
		end

		self.Icon:SetPoint("TOPLEFT", 88 + math.floor(adjustX / 2), -99 + math.floor(adjustY / 2));
	else			
		self.Icon:SetPoint("CENTER", self, "TOP", 0, -69);
		self.Icon:SetSize(64, 64);

		if useSquareBorder then -- square icon borders use atlases
			self.Icon:SetTexture(icon);

			self.IconBorder:ClearAllPoints();
			self.Icon:ClearAllPoints();
			self.Icon:SetPoint("TOPLEFT", 88, -99);

			self.IconBorder:SetAtlas("collections-itemborder-collected");
			self.IconBorder:SetTexCoord(0, 1, 0, 1);
			self.IconBorder:SetSize(80, 81);
			self.IconBorder:SetPoint("CENTER", self.Icon, "CENTER", 0, -3);
		else -- round icon borders use textures
			SetPortraitToTexture(self.Icon, icon);
			self.IconBorder:ClearAllPoints();

			self.Icon:ClearAllPoints();
			self.Icon:SetPoint("TOPLEFT", 88, -99);
			self.IconBorder:SetTexture("Interface\\Store\\Store-Splash");
			self.IconBorder:SetTexCoord(0.55957031, 0.79589844, 0.26269531, 0.51660156);
			self.IconBorder:SetSize(242, 260);
			self.IconBorder:SetPoint("TOPLEFT", 4, -4);
		end		
		self.IconBorder:Show();
	end
	self.Icon:Show();
	
	if self.GlowSpin and not overrideTexture then
		self.GlowSpin.SpinAnim:Play();
		self.GlowSpin:Show();
	elseif self.GlowSpin then
		self.GlowSpin.SpinAnim:Stop();
		self.GlowSpin:Hide();
	end

	if self.GlowPulse and not overrideTexture then
		self.GlowPulse.PulseAnim:Play();
		self.GlowPulse:Show();
	elseif self.GlowPulse then
		self.GlowPulse.PulseAnim:Stop();
		self.GlowPulse:Hide();
	end
end

function FullStoreCardMixin:Layout()
	self:SetSize(576, 471);
	
	self.Shadows:ClearAllPoints();
	self.Shadows:SetTexture("Interface\\Store\\Store-Main");
	self.Shadows:SetTexCoord(0.84375000, 0.97851563, 0.29980469, 0.37011719);
	self.Shadows:SetSize(138, 72);
	self.Shadows:SetPoint("TOPLEFT", 4, -4);

	self.Icon:ClearAllPoints();
	self.Icon:SetSize(68, 68);
	self.Icon:SetPoint("TOPLEFT", 88, -99);

	self.IconBorder:ClearAllPoints();
	self.IconBorder:SetTexture("Interface\\Store\\Store-Splash");
	self.IconBorder:SetTexCoord(0.55957031, 0.79589844, 0.26269531, 0.51660156);
	self.IconBorder:SetSize(242, 260);
	self.IconBorder:SetPoint("TOPLEFT", 4, -4);

	self.UpgradeArrow:ClearAllPoints();
	self.UpgradeArrow:SetPoint("CENTER", self.IconBorder, "CENTER", 18, -26);

	self.ProductName:ClearAllPoints();
	self.ProductName:SetFontObject("GameFontNormalWTF2");
	self.ProductName:SetJustifyV("TOP");
	self.ProductName:SetShadowOffset(1, -1);
	self.ProductName:SetShadowColor(0, 0, 0, 0);

	self.Description:ClearAllPoints();
	self.Description:SetFontObject("GameFontNormalLarge");
	self.Description:SetJustifyH("LEFT");
	self.Description:SetJustifyV("TOP");
	self.Description:SetTextColor(1, 0.84, 0.55);

	self.CurrentMarketPrice:ClearAllPoints();
	self.CurrentMarketPrice:SetSize(340, 0);
	self.CurrentMarketPrice:SetFontObject("GameFontNormalMed2");
	self.CurrentMarketPrice:SetJustifyH("LEFT");
	self.CurrentMarketPrice:SetPoint("TOPLEFT", self.Description, "BOTTOMLEFT", 0, -28);
	self.CurrentMarketPrice:SetTextColor(0.733, 0.588, 0.31);

	self.CurrentPrice:ClearAllPoints();
	self.CurrentPrice:SetFontObject("GameFontNormalLarge2");
	self.CurrentPrice:SetTextColor(1.0, 0.82, 0);

	self.NormalPrice:ClearAllPoints();
	self.NormalPrice:SetFontObject("GameFontNormalMed3");
	self.NormalPrice:SetTextColor(0.8, 0.66, 0);

	self.SalePrice:ClearAllPoints();
	self.SalePrice:SetFontObject("GameFontNormalLarge2");
	self.SalePrice:SetPoint("BOTTOMLEFT", self.NormalPrice, "BOTTOMRIGHT", 4, -1);
	self.SalePrice:SetTextColor(0.1, 1.0, 0.1);

	self.SplashBanner:ClearAllPoints();
	self.SplashBanner:SetSize(374, 77);
	self.SplashBanner:SetPoint("TOP", 3, 2);
	self.SplashBanner:Show();
	
	self.SplashBannerText:ClearAllPoints();
	self.SplashBannerText:SetPoint("CENTER", self.SplashBanner, "CENTER", 0, 16);
	self.SplashBannerText:SetTextColor(0.36, 0.18, 0.05);
	self.SplashBannerText:Show();

	self.Strikethrough:ClearAllPoints();
	self.Strikethrough:SetPoint("TOPLEFT", self.NormalPrice, "TOPLEFT", 0, 0);
	self.Strikethrough:SetPoint("BOTTOMRIGHT", self.NormalPrice, "BOTTOMRIGHT", 0, 0);

	self.GlowSpin:ClearAllPoints();
	self.GlowSpin:SetSize(253, 256);
	self.GlowSpin:SetPoint("CENTER", self.IconBorder, "CENTER", 0, 0);

	self.GlowPulse:ClearAllPoints();
	self.GlowPulse:SetSize(145, 138);
	self.GlowPulse:SetPoint("CENTER", self.IconBorder, "CENTER", 0, 0);

	self.BannerFadeIn:ClearAllPoints();
	self.BannerFadeIn:SetSize(374, 77);
	self.BannerFadeIn:SetPoint("TOP", 3, 2);

	self.ModelScene:ClearAllPoints();
	self.ModelScene:SetPoint("TOPLEFT", 8, -8);
	self.ModelScene:SetPoint("BOTTOMRIGHT", -8, 8);
	self.ModelScene:SetViewInsets(20, 400, 136, 180);

	self.InvisibleMouseOverFrame:ClearAllPoints();
	self.InvisibleMouseOverFrame:SetSize(68, 68);
	self.InvisibleMouseOverFrame:SetPoint("TOPLEFT", 86, -96);

	self.BuyButton:ClearAllPoints();
	self.BuyButton:SetSize(140, 30);
	self.BuyButton:SetPoint("BOTTOM", 0, 20);
	
	self.Magnifier:ClearAllPoints();
	self.Magnifier:SetSize(31, 35);
	self.Magnifier:SetPoint("LEFT", self.Shadows, "BOTTOMRIGHT", -40, 20);
	
	self.Checkmark:ClearAllPoints();
	self.Checkmark:SetSize(27, 27);
	self.Checkmark:SetPoint("BOTTOM", self.Magnifier, "TOP", 5, 2);	
end

function FullStoreCardMixin:SetStyle(overrideBackground)
	self.Card:ClearAllPoints();
	
	self.DiscountMiddle:Hide();
	self.DiscountLeft:Hide();
	self.DiscountRight:Hide();
	self.DiscountText:Hide();
	
	if overrideBackground then
		self.Card:SetPoint("CENTER");
		self.Card:SetAtlas(overrideBackground, true);
		self.Card:SetTexCoord(0, 1, 0, 1);
	else
		self.Card:SetPoint("TOPLEFT");
		self.Card:SetPoint("BOTTOMRIGHT");
		self.Card:SetTexture("Interface\\Store\\Store-Main");
		self.Card:SetTexCoord(0.00097656, 0.56347656, 0.00097656, 0.46093750);
	end
end


--------------------------------------------------
-- HORIZONTAL FULL STORE CARD MIXIN
HorizontalFullStoreCardMixin = CreateFromMixins(FullStoreCardMixin);

function HorizontalFullStoreCardMixin:SetStyle(overrideBackground)
	FullStoreCardMixin.SetStyle(self, overrideBackground);
	
	self.SplashBanner:Hide();
	self.SplashBannerText:Hide();
	self.ModelScene:SetViewInsets(20, 20, 20, 200);

	self.ProductName:ClearAllPoints();
	self.CurrentPrice:ClearAllPoints();
	self.NormalPrice:ClearAllPoints();
	self.Description:ClearAllPoints();

	if not self.ProductName.SetFontObjectsToTry then
		SecureMixin(self.ProductName, ShrinkUntilTruncateFontStringMixin);
	end
	self.ProductName:SetWidth(535);
	self.ProductName:SetMaxLines(1);
	self.ProductName:SetPoint("CENTER", 0, -32);
	self.ProductName:SetJustifyH("CENTER");
	self.ProductName:SetFontObjectsToTry("Game30Font", "GameFontNormalHuge2", "GameFontNormalLarge2");

	self.CurrentPrice:SetPoint("TOP", self.ProductName, "BOTTOM", 0, -6);

	local normalWidth = self.NormalPrice:GetStringWidth();
	local totalWidth = normalWidth + self.SalePrice:GetStringWidth();
	self.NormalPrice:SetPoint("TOP", self.ProductName, "BOTTOM", (normalWidth - totalWidth) / 2, -9);

	self.Description:SetPoint("TOP", self.CurrentPrice, "BOTTOM", 0, -12);
	self.Description:SetFontObject("GameFontNormalMed1");
	self.Description:SetWidth(490);
	self.Description:SetJustifyH("CENTER");

	self.SalePrice:SetFontObject("GameFontNormalLarge2");

	self.BuyButton:ClearAllPoints();
	self.BuyButton:SetPoint("BOTTOM", 0, 33);

	self.Magnifier:ClearAllPoints();
	self.Magnifier:SetPoint("TOPLEFT", self.Card, "TOPLEFT", 8, -8);

	self.Checkmark:ClearAllPoints();
	self.Checkmark:SetPoint("LEFT", self.Magnifier, "RIGHT", 9, 0);
	self.Checkmark:Hide();
end

--------------------------------------------------
-- VERTICAL FULL STORE CARD MIXIN
VerticalFullStoreCardMixin = CreateFromMixins(FullStoreCardMixin);

function VerticalFullStoreCardMixin:SetStyle(overrideBackground)
	FullStoreCardMixin.SetStyle(self, overrideBackground);
	
	self.SplashBanner:Show();
	self.SplashBannerText:Show();
	self.ModelScene:SetViewInsets(20, 400, 136, 180);

	self.ProductName:ClearAllPoints();
	self.CurrentPrice:ClearAllPoints();
	self.NormalPrice:ClearAllPoints();
	self.Description:ClearAllPoints();

	if not self.ProductName.SetFontObjectsToTry then
		SecureMixin(self.ProductName, ShrinkUntilTruncateFontStringMixin);
	end
	self.ProductName:SetWidth(300);
	self.ProductName:SetMaxLines(1);
	self.ProductName:SetPoint("TOPLEFT", self, "TOP", -83, -70);
	self.ProductName:SetJustifyH("LEFT");
	self.ProductName:SetFontObjectsToTry("GameFontNormalWTF2", "Game30Font", "GameFontNormalHuge3");

	self.CurrentPrice:SetPoint("TOPLEFT", self.Description, "BOTTOMLEFT", 0, -28);

	self.NormalPrice:SetPoint("TOPLEFT", self.Description, "BOTTOMLEFT", 0, -28);

	self.Description:SetPoint("TOPLEFT", self.ProductName, "BOTTOMLEFT", 0, -16);
	self.Description:SetFontObject("GameFontNormalLarge");
	self.Description:SetWidth(340);
	self.Description:SetJustifyH("LEFT");

	self.SalePrice:SetFontObject("GameFontNormalLarge2");

	self.BuyButton:ClearAllPoints();
	self.BuyButton:SetPoint("TOPLEFT", self.CurrentPrice, "BOTTOMLEFT", 0, -20);

	self.Magnifier:ClearAllPoints();
	self.Magnifier:SetPoint("LEFT", self.Shadows, "BOTTOMRIGHT", -40, 20);

	self.Checkmark:ClearAllPoints();
	self.Checkmark:SetPoint("BOTTOM", self.Magnifier, "TOP", 5, 2);
end
