local PVPMatchResultsStyle = {
	PanelColors = {
			CreateColor(0.52, 0.075, 0.18),	-- Horde
			CreateColor(0.72, 0.37, 1.0),	-- Horde Alternate
			CreateColor(0, .32, .89),		-- Alliance
			CreateColor(0.85, 0.71, 0.26),	-- Alliance Alternate
	},
	Theme = {
		Horde = {
			decoratorOffsetY = -37,
			Decorator = "scoreboard-horde-header",
			nineSliceLayout = "BFAMissionHorde",
		},
		Alliance = {
			decoratorOffsetY = -28,
			Decorator = "scoreboard-alliance-header",
			nineSliceLayout = "BFAMissionAlliance",
		},
	},
}

local ACTIVE_EVENTS = {
	"PVP_MATCH_COMPLETE",
	"TREASURE_PICKER_CACHE_FLUSH",
	"LFG_ROLE_CHECK_DECLINED",
	"POST_MATCH_ITEM_REWARD_UPDATE",
	"POST_MATCH_CURRENCY_REWARD_UPDATE",
	"LFG_ROLE_CHECK_SHOW",
	"LFG_READY_CHECK_DECLINED",
	"LFG_READY_CHECK_SHOW",
};

function PVPMatchResultsStyle.GetPanelColor(faction, useAlternateColor)
	local index = PVPMatchUtil.GetColorIndex(faction, useAlternateColor);
	return PVPMatchResultsStyle.PanelColors[index];
end

local LeaveMatchFormatter = CreateFromMixins(SecondsFormatterMixin);
LeaveMatchFormatter:OnLoad(0, SecondsFormatter.Abbreviation.OneLetter, true);
LeaveMatchFormatter:SetStripIntervalWhitespace(true);
function LeaveMatchFormatter:GetDesiredUnitCount(seconds)
	return 1;
end
function LeaveMatchFormatter:GetMinInterval(seconds)
	return SecondsFormatter.Interval.Seconds;
end

PVPMatchResultsMixin = {};
function PVPMatchResultsMixin:OnLoad()
	self:RegisterEvent("PVP_MATCH_ACTIVE");
	self:RegisterEvent("PVP_MATCH_INACTIVE");
	
	local tabContainer = self.content.tabContainer;
	self.scrollFrame = self.content.scrollFrame;
	self.scrollFrameBackground = self.scrollFrame.background;
	self.scrollCategories = self.content.scrollCategories;
	self.tabGroup = tabContainer.tabGroup;
	self.tab1 = self.tabGroup.tab1;
	self.tab2 = self.tabGroup.tab2;
	self.tab3 = self.tabGroup.tab3;
	self.leaveButton = self.buttonContainer.leaveButton;
	self.requeueButton = self.buttonContainer.requeueButton;
	local matchTimeContainer = tabContainer.matchTimeContainer;
	self.matchTimeValue = matchTimeContainer.matchTimeValue;
	self.earningsContainer = self.content.earningsContainer;
	self.rewardsContainer = self.earningsContainer.rewardsContainer;
	self.rewardsHeader = self.rewardsContainer.header;
	self.itemContainer = self.rewardsContainer.items;
	self.progressContainer = self.earningsContainer.progressContainer;
	self.progressHeader = self.progressContainer.header;
	self.honorFrame = self.progressContainer.honor;
	self.honorText = self.honorFrame.text;
	self.honorButton = self.honorFrame.button;
	self.conquestFrame = self.progressContainer.conquest;
	self.conquestText = self.conquestFrame.text;
	self.conquestButton = self.conquestFrame.button;
	self.ratingFrame = self.progressContainer.rating;
	self.ratingText = self.progressContainer.rating.text;
	self.ratingButton = self.progressContainer.rating.button;
	self.earningsArt = self.content.earningsArt;
	self.earningsBackground = self.earningsArt.background;
	self.tintFrames = {self.glowTop, self.earningsBackground, self.scrollFrameBackground};
	self.rewardFrames = {self.honorFrame, self.conquestFrame, self.ratingFrame};

	self.earningsContainer:Hide();
	self.progressHeader:SetText(PVP_PROGRESS_REWARDS_HEADER);
	self.rewardsHeader:SetText(PVP_ITEM_REWARDS_HEADER);
	matchTimeContainer.matchTime:SetText(PVP_MATCH_TIME);

	self.conquestButton:SetTooltipAnchor("ANCHOR_RIGHT");
	self.requeueButton:SetScript("OnClick", function() self:OnRequeueButtonClicked(self.requeueButton) end);
	self.requeueButton:SetText(PVP_QUEUE_AGAIN);
	
	self.leaveButton:SetScript("OnClick", function() self:OnLeaveButtonClicked(self.leaveButton) end);

	self.tab1:SetText(ALL);
	self.Tabs = {self.tab1, self.tab2, self.tab3};
	PanelTemplates_SetNumTabs(self, #self.Tabs);
	for k, tab in pairs(self.Tabs) do
		tab:SetScript("OnClick", function() self:OnTabGroupClicked(tab) end);
	end
	PanelTemplates_SetTab(self, 1);

	HybridScrollFrame_OnLoad(self.scrollFrame);
	HybridScrollFrame_CreateButtons(self.scrollFrame, "PVPTableRowTemplate");
	HybridScrollFrame_SetDoNotHideScrollBar(self.scrollFrame, true);

	self.itemPool = CreateFramePool("BUTTON", self.itemContainer, "PVPMatchResultsLoot");
	self.tableBuilder = CreateTableBuilder(HybridScrollFrame_GetButtons(self.scrollFrame));
	self.tableBuilder:SetHeaderContainer(self.scrollCategories);
end

function PVPMatchResultsMixin:Init(winner, duration)
	if self.isInitialized then
		return;
	end
	self.isInitialized = true;

	local factionIndex = GetBattlefieldArenaFaction();
	local enemyFactionIndex = (factionIndex+1)%2;
	local GetOutcomeText = function()
		if winner == factionIndex then
			return PVP_MATCH_VICTORY;
		elseif winner == enemyFactionIndex then
			return PVP_MATCH_DEFEAT;
		else
			return PVP_MATCH_DRAW;
		end
	end
	self.header:SetText(GetOutcomeText());

	-- Using reference text size before adding a margin sufficiently sized
	-- to allow space for the remaining time.
	self.leaveButton:SetText(PVP_MATCH_LEAVE_BUTTON);
	local baseWidth = self.leaveButton:GetFontString():GetStringWidth();
	local timeWidthMargin = 100;
	self.leaveButton:SetWidth(baseWidth + timeWidthMargin);
	self.buttonContainer:MarkDirty();

	self.UpdateLeaveButton = function()
		local shutdownTime = GetBattlefieldInstanceExpiration()/1000;
		if shutdownTime > 0 then
			local formattedTime = LeaveMatchFormatter:Format(shutdownTime);
			self.leaveButton:SetText(PVP_LEAVE_BUTTON_TIME:format(PVP_MATCH_LEAVE_BUTTON, formattedTime));
		else
			self.leaveButton:SetText(PVP_MATCH_LEAVE_BUTTON);
		end
	end;

	local isArenaSkirmish = IsArenaSkirmish();
	self.requeueButton:SetEnabled(isArenaSkirmish);
	self.requeueButton:SetShown(isArenaSkirmish);
	
	local isArena = IsActiveBattlefieldArena();
	local isLFD = IsInLFDBattlefield();
	local isFactionalMatch = not (isArena or isArenaSkirmish or isLFD);

	-- The border is related to the character's faction, regardless of the
	-- outcome of the match.
	local factionGroup = UnitFactionGroup("player");
	local teamIndex = (factionGroup and factionGroup == "Horde") and 0 or 1;
	self:SetupBorder(teamIndex, isFactionalMatch);
	
	local formattedTime = PVPMatchUtil.MatchTimeFormatter:Format(duration);
	self.matchTimeValue:SetText(PVP_TIME_ELAPSED:format(formattedTime));

	if isFactionalMatch then
		local teamInfos = { 
			C_PvP.GetTeamInfo(0),
			C_PvP.GetTeamInfo(1), 
		};
		self.tab2:SetText(PVP_TAB_FILTER_COUNTED:format(FACTION_ALLIANCE, teamInfos[2].size));
		self.tab3:SetText(PVP_TAB_FILTER_COUNTED:format(FACTION_HORDE, teamInfos[1].size));
		PanelTemplates_ResizeTabsToFit(self, 600);
	end
	self.tabGroup:SetShown(isFactionalMatch);

	local useAlternateColor = not isFactionalMatch;
	local r, g, b = PVPMatchResultsStyle.GetPanelColor(factionIndex, useAlternateColor):GetRGB();
	for k, frame in pairs(self.tintFrames) do
		frame:SetVertexColor(r, g, b);
	end

	local buttons = HybridScrollFrame_GetButtons(self.scrollFrame);
	for k, button in pairs(buttons) do
		button:Init(useAlternateColor);
	end

	ConstructPVPMatchTable(self.tableBuilder, PVPMatchUtil.IsRatedBattleground(), isArena, isLFD, useAlternateColor);
	
	-- 820FIXME IMPLEMENT NAME DROPDOWN
end
function PVPMatchResultsMixin:TryDisplayRewards()
	if self.haveConquestData and self.hasRewardTimerElapsed then
		self:DisplayRewards();
	end
end
function PVPMatchResultsMixin:Reset()
	self.hasRewardTimerElapsed = false;
	self.rewardTimer = false;
	self.earningsContainer:Hide();
end
function PVPMatchResultsMixin:OnEvent(event, ...)
	if event == "PVP_MATCH_ACTIVE" then
		FrameUtil.RegisterFrameForEvents(self, ACTIVE_EVENTS);
		self:Reset();
	elseif event == "PVP_MATCH_COMPLETE" then
		local winner, duration = ...;
		self:TryShow(winner, duration);
	elseif event == "TREASURE_PICKER_CACHE_FLUSH" then
		self.haveConquestData = self:HaveConquestData();
		if not haveConquestData then
			self:RegisterEvent("QUEST_LOG_UPDATE");
		end
	elseif event == "POST_MATCH_ITEM_REWARD_UPDATE" or event == "POST_MATCH_CURRENCY_REWARD_UPDATE" then
		if self.hasRewardTimerElapsed then
			-- 820FIXME We've received an item reward late. Reinitialize the reward section. This is a stopgap
			-- until we've resolved the complete item details as part of the match complete message.
			self:TryDisplayRewards();
		end
	elseif event == "QUEST_LOG_UPDATE" then
		self.haveConquestData = self:HaveConquestData();
		if self.haveConquestData then
			self:UnregisterEvent("QUEST_LOG_UPDATE");
			self:TryDisplayRewards();
		end
	elseif event == "PVP_MATCH_INACTIVE" then
		FrameUtil.UnregisterFrameForEvents(self, ACTIVE_EVENTS);
		self:UnregisterEvent("QUEST_LOG_UPDATE");
		self.isInitialized = false;

		HideUIPanel(self);
	elseif event == "LFG_ROLE_CHECK_DECLINED" or event == "LFG_READY_CHECK_DECLINED" then
		self.requeueButton:Enable();
	elseif event == "LFG_ROLE_CHECK_SHOW" or event == "LFG_READY_CHECK_SHOW" then
		self.requeueButton:Disable();
	end
end
function PVPMatchResultsMixin:TryShow(winner, duration)
	-- Get the conquest information if necessary. This will normally be cached
	-- at the beginning of the match, but this is to deal with any rare cases
	-- where the sparse item or treasure picker db's have been flushed on us.
	self.haveConquestData = self:HaveConquestData();
	if not self.haveConquestData then
		self:RegisterEvent("QUEST_LOG_UPDATE");
	end

	-- 820FIXME The items earned are not known at match completion time. This is a stopgap
	-- until we've resolved the complete item details as part of the match complete message.
	if not self.hasRewardTimerElapsed and not self.rewardTimer then
		self.rewardTimer = C_Timer.NewTimer(1.0, 
			function()
				self.rewardTimer = nil;
				self.hasRewardTimerElapsed = true;
				self:TryDisplayRewards();
			end
		);
	end

	self:Init(winner, duration);
	ShowUIPanel(self);
end
function PVPMatchResultsMixin:DisplayRewards()
	self.itemPool:ReleaseAll();

	for k, item in pairs(C_PvP.GetPostMatchItemRewards()) do
		-- Conquest is displayed in the progress section, so ignore it if found.
		if not (item.type == "currency" and C_CurrencyInfo.GetCurrencyIDFromLink(item.link) == Constant.Currency.Conquest) then
			self:AddItemReward(item);
		end
	end

	for k, frame in pairs(self.rewardFrames) do
		frame:Hide();
	end

	for k, currency in pairs(C_PvP.GetPostMatchCurrencyRewards()) do
		if currency.currencyType == Constant.Currency.Honor then
			self:InitHonorFrame(currency);
		elseif currency.currencyType == Constant.Currency.Conquest then
			self:InitConquestFrame(currency);
		end
	end
	
	-- Skirmish is considered rated, ignore it.
	if C_PvP.IsRatedMap() and not IsArenaSkirmish() then
		self:InitRatingFrame();
	end

	local previousItemFrame;
	for itemFrame in self.itemPool:EnumerateActive() do
		if previousItemFrame then
			itemFrame:SetPoint("TOPLEFT", previousItemFrame, "TOPRIGHT", 17, 0);
		else
			itemFrame:SetPoint("TOPLEFT");
		end

		itemFrame:Show();
		previousItemFrame = itemFrame;
	end
	
	local showItems = previousItemFrame ~= nil;
	self.rewardsContainer:SetShown(showItems);

	-- Visibility of the progress elements can be mixed, but are expected to be in the order of
	-- honor, then conquest, then rating.
	local progressFramesShown = {};
	local InsertIfShown = function(frame)
		if frame:IsShown() then
			tinsert(progressFramesShown, frame);
		end
	end;
	for k, frame in pairs(self.rewardFrames) do
		InsertIfShown(frame);
	end

	local previousProgressFrame;
	for i, progressFrame in ipairs(progressFramesShown) do
		if previousProgressFrame then
			progressFrame:SetPoint("LEFT", previousProgressFrame, "RIGHT", 22, 0);
		else
			progressFrame:SetPoint("TOPLEFT", self.progressHeader, "BOTTOMLEFT", 3, -11);
		end

		progressFrame:MarkDirty();
		previousProgressFrame = progressFrame;
	end
	
	local showProgress = previousProgressFrame ~= nil;
	self.progressContainer:SetShown(showProgress);
	if showProgress then
		if previousItemFrame then
			-- Anchor the progress rewards container to the item rewards container if items are present present, otherwise,
			-- the progress rewards will be centered when arranged by the resize layout frame.
			self.progressContainer:SetPoint("TOPLEFT", self.rewardsContainer, "TOPRIGHT", 50, 0);
		else
			self.progressContainer:SetPoint("TOPLEFT");
		end
	end
	
	if showItems or showProgress then
		self.earningsContainer:Show();
		self.earningsContainer:MarkDirty();
		self.earningsContainer.FadeInAnim:Play();
		self.earningsArt.BurstBgAnim:Play();

		local AddDelayToAnimations = function(delay, ...)
			for animIndex = 1, select("#", ...) do
				local anim = select(animIndex, ...);
				if not anim.initialStartDelay then
					anim.initialStartDelay = anim:GetStartDelay() or 0;
				end
				anim:SetStartDelay(anim.initialStartDelay + delay);
			end
		end

		-- Design ask is to mimic the garrision reward sequence. 
		-- 1.1 (chest animation) - .75 (background burst animation) == .35.
		local itemStartDelay = .35;
		for itemFrame in self.itemPool:EnumerateActive() do
			local animGroup = itemFrame.IconAnim;
			AddDelayToAnimations(itemStartDelay, animGroup:GetAnimations());
			animGroup:Play();
		end
	end
end

-- If this function returns false, it also comes with the side-effect of assigning
-- a callback and signalling QUEST_LOG_UPDATE. Unfortunately, this function has multiple steps
-- of failure, so we continue to call this until it succeeds after every QUEST_LOG_UPDATE event.
function PVPMatchResultsMixin:HaveConquestData()
	local conquestQuestID = select(3, PVPGetConquestLevelInfo());
	return HaveQuestRewardData(conquestQuestID);
end
function PVPMatchResultsMixin:OnUpdate()
	if self.UpdateLeaveButton then
		self:UpdateLeaveButton();		
	end

	local buttons = HybridScrollFrame_GetButtons(self.scrollFrame);
	local buttonCount = #buttons;
	local displayCount = GetNumBattlefieldScores();
	local buttonHeight = buttons[1]:GetHeight();
	local visibleElementHeight = displayCount * buttonHeight;

	local offset = HybridScrollFrame_GetOffset(self.scrollFrame);
	local populateCount = math.min(buttonCount, displayCount);
	self.tableBuilder:Populate(offset, populateCount);
	
	for i = 1, buttonCount do
		local visible = i <= displayCount;
		buttons[i]:SetShown(visible);
	end

	local regionHeight = self.scrollFrame:GetHeight();
	HybridScrollFrame_Update(self.scrollFrame, visibleElementHeight, regionHeight);
end
function PVPMatchResultsMixin:AddItemReward(item)
	local frame = self.itemPool:Acquire();

	-- Specialization icon is intended to be hidden.
	local unusedSpecID;
	local isCurrency = item.type == "currency";
	local isIconBorderShown = true;
	local isIconBorderDropShadowShown = true;
	frame:Init(item.link, item.quantity, unusedSpecID, isCurrency, item.isUpgraded, isIconBorderShown, isIconBorderDropShadowShown);
	frame:SetScale(.7931);
end
function PVPMatchResultsMixin:InitHonorFrame(currency)
	local deltaString = FormatValueWithSign(currency.quantityChanged);
	self.honorText:SetText(PVP_HONOR_CHANGE:format(deltaString));
	self.honorButton:Update();
	self.honorFrame:Show();
end
function PVPMatchResultsMixin:InitConquestFrame(currency)
	local deltaString = FormatValueWithSign(currency.quantityChanged / 100);
	self.conquestText:SetText(PVP_CONQUEST_CHANGE:format(deltaString));

	local questID = select(3, PVPGetConquestLevelInfo());
	if questID and IsPlayerAtEffectiveMaxLevel() then
		self.conquestButton:Setup(questID, nil);
	else
		self.conquestButton:Clear();
	end

	self.conquestFrame:Show();
end
function PVPMatchResultsMixin:InitRatingFrame()
	local localPlayerScoreInfo = C_PvP.GetScoreInfoByPlayerGuid(GetPlayerGuid());
	local ratingChange = localPlayerScoreInfo.ratingChange;
	local rating = localPlayerScoreInfo.rating;
	self.ratingButton:Init(rating, ratingChange);

	local personalRatedInfo = C_PvP.GetPVPActiveMatchPersonalRatedInfo();
	if personalRatedInfo then
		local tierInfo = C_PvP.GetPvpTierInfo(personalRatedInfo.tier);
		self.ratingButton:Setup(tierInfo, ranking);
	end
	local deltaString = FormatValueWithSign(ratingChange);
	self.ratingText:SetText(PVP_RATING_CHANGE:format(deltaString));

	self.ratingFrame:Show();
end
function PVPMatchResultsMixin:SetupBorder(factionIndex, isFactionalMatch)
	local theme = PVPMatchResultsStyle.Theme[factionIndex == 0 and "Horde" or "Alliance"];

	local themeDecoration = self.overlay.decorator;
	local showDecoration = not isFactionalMatch;
	if showDecoration then
		themeDecoration:SetPoint("BOTTOM", self, "TOP", 0, theme.decoratorOffsetY);
		themeDecoration:SetAtlas(theme.Decorator, true);
	end
	themeDecoration:SetShown(showDecoration);

	NineSliceUtil.ApplyLayoutByName(self, theme.nineSliceLayout);
end
function PVPMatchResultsMixin:OnLeaveButtonClicked(button)
	if IsInLFDBattlefield() then
		ConfirmOrLeaveLFGParty();
    else
		ConfirmOrLeaveBattlefield();
    end

	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION);
end
function PVPMatchResultsMixin:OnRequeueButtonClicked(button)
	button:Disable();
    RequeueSkirmish();

	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB);
end
function PVPMatchResultsMixin:OnClose()
	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_CLOSE);

	HideParentPanel(self);
end
function PVPMatchResultsMixin:OnHide()
	-- 820FIXME IMPLEMENT NAME DROPDOWN
	--CloseDropDownMenus();
end
function PVPMatchResultsMixin:OnTabGroupClicked(tab)
	PanelTemplates_SetTab(self, tab:GetID());
	SetBattlefieldScoreFaction(tab.factionEnum);
	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB);
end

PVPMatchResultsRatingMixin = {};
function PVPMatchResultsRatingMixin:Init(rating, ratingChange)
	self.rating = rating;
	self.ratingChange = ratingChange;
	self.ratingNew = rating + ratingChange;

	local teamInfos = { 
		C_PvP.GetTeamInfo(0),
		C_PvP.GetTeamInfo(1), 
	};
	local factionIndex = GetBattlefieldArenaFaction();
	self.friendlyMMR = BATTLEGROUND_YOUR_AVERAGE_RATING:format(teamInfos[factionIndex+1].ratingMMR);
	local enemyFactionIndex = (factionIndex+1)%2;
	self.enemyMMR = BATTLEGROUND_ENEMY_AVERAGE_RATING:format(teamInfos[enemyFactionIndex+1].ratingMMR);
end
function PVPMatchResultsRatingMixin:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip_SetTitle(GameTooltip, PVP_RATING_HEADER);
	GameTooltip_AddNormalLine(GameTooltip, PVP_RATING_PREVIOUS:format(self.rating));

	local deltaString = FormatValueWithSign(self.ratingChange);
	GameTooltip_AddNormalLine(GameTooltip, PVP_RATING_GAINED:format(deltaString));
	GameTooltip_AddNormalLine(GameTooltip, PVP_RATING_CURRENT:format(self.ratingNew));
	GameTooltip_AddBlankLineToTooltip(GameTooltip);

	GameTooltip_AddNormalLine(GameTooltip, self.friendlyMMR);
	GameTooltip_AddNormalLine(GameTooltip, self.enemyMMR);
	
	GameTooltip:Show();
end
function PVPMatchResultsRatingMixin:OnLeave()
	GameTooltip:Hide();
end

function TogglePVPMatchResultsFrame()
	if ( PVPMatchResults:IsShown() ) then
		HideUIPanel(PVPMatchResults);
	else
		PVPMatchResults:TryShow(C_PvP.GetActiveMatchWinner(), C_PvP.GetActiveMatchDuration());
	end
end